@@ -64,12 +64,6 @@ int main(void)
 #define MSG(...)  printf("[cookies dpi]: " __VA_ARGS__)
 
 
-/* This one is tricky, some sources state it should include the byte
- * for the terminating NULL, and others say it shouldn't. */
-# define D_SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) \
-                        + strlen ((ptr)->sun_path))
-
-
 /*
  * a_List_add()
  *
@@ -184,14 +178,19 @@ static FILE *Cookies_fopen(const char *filename, const char *mode,
                            char *init_str)
 {
    FILE *F_in;
-   int fd;
+   int fd, rc;
 
    if ((F_in = fopen(filename, mode)) == NULL) {
       /* Create the file */
       fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
       if (fd != -1) {
-         if (init_str)
-            write(fd, init_str, strlen(init_str));
+         if (init_str) {
+            rc = write(fd, init_str, strlen(init_str));
+            if (rc == -1) {
+               MSG("Cookies: Could not write initial string to file %s: %s\n",
+                  filename, dStrerror(errno));
+            }
+         }
          close(fd);
 
          MSG("Created file: %s\n", filename);
@@ -228,7 +227,7 @@ static void Cookies_free_cookie(CookieData_t *cookie)
 static void Cookies_init()
 {
    CookieData_t *cookie;
-   char *filename;
+   char *filename, *rc = NULL;
    char line[LINE_MAXLEN];
 #ifndef HAVE_LOCKF
    struct flock lck;
@@ -279,7 +278,12 @@ static void Cookies_init()
    /* Get all lines in the file */
    while (!feof(file_stream)) {
       line[0] = '\0';
-      fgets(line, LINE_MAXLEN, file_stream);
+      rc = fgets(line, LINE_MAXLEN, file_stream);
+      if (!rc && ferror(file_stream)) {
+         MSG("Cookies1: Error while reading from cookies.txt: %s\n",
+             dStrerror(errno));
+         break; /* bail out */
+      }
 
       /* Remove leading and trailing whitespaces */
       dStrstrip(line);
@@ -344,7 +348,12 @@ static void Cookies_init()
       /* Get all lines in the file */
       while (!feof(old_cookies_file_stream)) {
          line[0] = '\0';
-         fgets(line, LINE_MAXLEN, old_cookies_file_stream);
+         rc = fgets(line, LINE_MAXLEN, old_cookies_file_stream);
+         if (!rc && ferror(old_cookies_file_stream)) {
+            MSG("Cookies2: Error while reading from cookies file: %s\n",
+                dStrerror(errno));
+            break; /* bail out */
+         }
 
          /* Remove leading and trailing whitespaces */
          dStrstrip(line);
@@ -431,8 +440,9 @@ static void Cookies_save_and_free()
 
    rewind(file_stream);
    fd = fileno(file_stream);
-   ftruncate(fd, 0);
-   fprintf(file_stream, cookies_txt_header_str);
+   if (ftruncate(fd, 0) == -1)
+      MSG("Cookies: Truncate file stream failed: %s\n", dStrerror(errno));
+   fprintf(file_stream, "%s", cookies_txt_header_str);
 
    /* Iterate cookies per domain, saving and freeing */
    while ((node = dList_nth_data(cookies, 0))) {
@@ -545,7 +555,7 @@ static time_t Cookies_create_timestamp(const char *expires)
    }
 
    /* Error checks  --this may be overkill */
-   if (!(day > 0 && day < 32 && month > 0 && month < 13 && year > 1970 &&
+   if (!(day > 0 && day < 32 && month > 0 && month < 13 && year >= 1970 &&
          hour >= 0 && hour < 24 && minutes >= 0 && minutes < 60 &&
          seconds >= 0 && seconds < 60)) {
       MSG("%s%s\n", E_msg, expires);
@@ -909,6 +919,9 @@ static CookieData_t *Cookies_parse_one(int url_port, char **cookie_str)
          }
       } else if (dStrcasecmp(attr, "Secure") == 0) {
          cookie->secure = TRUE;
+      } else if (dStrcasecmp(attr, "HttpOnly") == 0) {
+         // this case is intentionally left blank, because we do not
+         // do client-side scripting (yet).
       } else {
          /* Oops! this can't be good... */
          MSG("Cookie contains unknown attribute: '%s'\n", attr);
@@ -1216,7 +1229,7 @@ static int Cookie_control_init(void)
 {
    CookieControl cc;
    FILE *stream;
-   char *filename;
+   char *filename, *rc;
    char line[LINE_MAXLEN];
    char domain[LINE_MAXLEN];
    char rule[LINE_MAXLEN];
@@ -1234,7 +1247,12 @@ static int Cookie_control_init(void)
    /* Get all lines in the file */
    while (!feof(stream)) {
       line[0] = '\0';
-      fgets(line, LINE_MAXLEN, stream);
+      rc = fgets(line, LINE_MAXLEN, stream);
+      if (!rc && ferror(stream)) {
+         MSG("Cookies3: Error while reading rule from cookiesrc: %s\n",
+             dStrerror(errno));
+         break; /* bail out */
+      }
 
       /* Remove leading and trailing whitespaces */
       dStrstrip(line);
@@ -1244,12 +1262,11 @@ static int Cookie_control_init(void)
          j = 0;
 
          /* Get the domain */
-         while (!isspace(line[i]))
+         while (line[i] != '\0' && !isspace(line[i]))
             domain[j++] = line[i++];
          domain[j] = '\0';
 
          /* Skip past whitespaces */
-         i++;
          while (isspace(line[i]))
             i++;
 
