@@ -107,7 +107,7 @@ class DLItem {
    enum {
       ST_newline, ST_number, ST_discard, ST_copy
    };
- 
+
    pid_t mPid;
    int LogPipe[2];
    char *shortname, *fullname;
@@ -291,10 +291,11 @@ static void prButton_scb(Widget *, void *cb_data)
 DLItem::DLItem(const char *full_filename, const char *url, DLAction action)
 {
    struct stat ss;
-   char *p, *esc_url;
+   const char *p;
+   char *esc_url;
 
    if (pipe(LogPipe) < 0) {
-      MSG("pipe, %s\n", strerror(errno));
+      MSG("pipe, %s\n", dStrerror(errno));
       return;
    }
    /* Set FD to background */
@@ -513,7 +514,7 @@ void DLItem::log_text_add(const char *buf, ssize_t st)
          if (isdigit(*q++ = *p)) {
             // keep here
          } else if (*p == 'K') {
-            for(--q; isdigit(q[-1]); --q); log_state = ST_discard;
+            for (--q; isdigit(q[-1]); --q) ; log_state = ST_discard;
          } else {
             log_state = ST_copy;
          }
@@ -685,7 +686,7 @@ void DLItem::update()
 
    /* Update curr_size */
    if (stat(fullname, &ss) == -1) {
-      MSG("stat, %s\n", strerror(errno));
+      MSG("stat, %s\n", dStrerror(errno));
       return;
    }
    update_size((int)ss.st_size);
@@ -744,7 +745,7 @@ void DLItem::update()
 /*! SIGCHLD handler
  */
 static void raw_sigchld(int)
-{          
+{
    caught_sigchld = 1;
 }
 
@@ -833,7 +834,7 @@ static ssize_t readline(int socket, Dstr ** msg)
    while (st < 0 && errno == EINTR);
 
    if (st == -1)
-      MSG("readline, %s\n", strerror(errno));
+      MSG("readline, %s\n", dStrerror(errno));
 
    dStr_truncate(*msg, 0);
    if (st > 0)
@@ -888,7 +889,7 @@ static void read_req_cb(int req_fd, void *)
       new_socket = accept(req_fd, (struct sockaddr *) &clnt_addr, &csz);
    } while (new_socket == -1 && errno == EINTR);
    if (new_socket == -1) {
-      MSG("accept, %s fd=%d\n", strerror(errno), req_fd);
+      MSG("accept, %s fd=%d\n", dStrerror(errno), req_fd);
       return;
    }
 
@@ -976,7 +977,7 @@ void DLWin::add(const char *full_filename, const char *url, DLAction action)
    } else if (f_pid < 0) {
       perror("fork, ");
       exit(1);
-   } else {   
+   } else {
       /* father */
       dl_win->show();
       dl_item->pid(f_pid);
@@ -999,7 +1000,7 @@ DLAction DLWin::check_filename(char **p_fullname)
       return DL_NEWFILE;
 
    ds = dStr_sized_new(128);
-   dStr_sprintf(ds, 
+   dStr_sprintf(ds,
                 "The file:\n  %s (%d Bytes)\nalready exists. What do we do?",
                 *p_fullname, (int)ss.st_size);
    ch = fltk::choice(ds->str, "Rename", "Continue", "Abort");
@@ -1020,18 +1021,14 @@ DLAction DLWin::check_filename(char **p_fullname)
 }
 
 /*
- * Add a new download request to the main window and
- * fork a child to do the job.
+ * Delete a download request from the main window.
  */
 void DLWin::del(int n_item)
 {
    DLItem *dl_item = mDList->get(n_item);
 
-   // Remove the widget from the scroll group
+   // Remove the widget from the packed group
    mPG->remove(dl_item->get_widget());
-   // Resize the scroll group
-   mPG->resize(mWin->w(), 1);
-
    mDList->del(n_item);
    delete(dl_item);
 }
@@ -1093,7 +1090,7 @@ DLWin::DLWin(int ww, int wh) {
     mScroll->end();
     mScroll->type(ScrollGroup::VERTICAL);
    mWin->end();
-   mWin->resizable(mPG);
+   mWin->resizable(mScroll);
    mWin->callback(dlwin_esc_cb, NULL);
    mWin->show();
 
