@@ -3,16 +3,15 @@
  *
  * This server is an example. Play with it and modify to your taste.
  *
- * Copyright 2003 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2003-2007 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  */
 
-#include <glib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <stdio.h>
@@ -22,6 +21,12 @@
 #include "../dpip/dpip.h"
 #include "dpiutil.h"
 
+/*
+ * Debugging macros
+ */
+#define _MSG(...)
+#define MSG(...)  printf("[hello dpi]: " __VA_ARGS__)
+
 /*---------------------------------------------------------------------------*/
 
 
@@ -32,31 +37,31 @@ int main(void)
 {
    FILE *in_stream;
    SockHandler *sh;
-   gchar *dpip_tag, *cmd = NULL, *url = NULL, *child_cmd = NULL;
-   gchar *esc_tag, *d_cmd;
+   char *dpip_tag, *cmd = NULL, *url = NULL, *child_cmd = NULL;
+   char *esc_tag, *d_cmd;
    size_t n;
-   gint ret;
-   gchar buf[4096];
-   gchar *choice[] = {"Window was closed", "Yes", "No",
+   int ret;
+   char buf[4096];
+   char *choice[] = {"Window was closed", "Yes", "No",
                       "Could be", "It's OK", "Cancel"};
                    /* "Could>be", ">It's OK", "Can'>cel"};  --for testing */
-   gint choice_num;
+   int choice_num;
 
-   g_printerr("hello.dpi:: starting...\n");
+   MSG("starting...\n");
 
    /* Initialize the SockHandler */
    sh = sock_handler_new(STDIN_FILENO, STDOUT_FILENO, 2*1024);
 
    /* Read the dpi command from STDIN */
    dpip_tag = sock_handler_read(sh);
-   g_printerr("[%s]\n", dpip_tag);
+   MSG("tag = [%s]\n", dpip_tag);
 
    cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
    url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
 
 /*-- Dialog part */
 {
-   gchar *dpip_tag2, *dialog_msg;
+   char *dpip_tag2, *dialog_msg;
 
    /* Let's confirm the request */
    /* NOTE: you can send less alternatives (e.g. only alt1 and alt2) */
@@ -64,12 +69,12 @@ int main(void)
               "cmd=%s msg=%s alt1=%s alt2=%s alt3=%s alt4=%s alt5=%s",
               "dialog", "Do you want to see the hello page?",
               choice[1], choice[2], choice[3], choice[4], choice[5]);
-   sock_handler_write_str(sh, d_cmd, 1);
-   g_free(d_cmd);
+   sock_handler_write_str(sh, 1, d_cmd);
+   dFree(d_cmd);
 
    /* Get the answer */
    dpip_tag2 = sock_handler_read(sh);
-   g_printerr("[%s]\n", dpip_tag2);
+   MSG("tag = [%s]\n", dpip_tag2);
 
    /* Get "msg" value */
    dialog_msg = a_Dpip_get_attr(dpip_tag2, strlen(dpip_tag2), "msg");
@@ -77,15 +82,15 @@ int main(void)
    if (dialog_msg)
       choice_num = *dialog_msg - '0';
 
-   g_free(dialog_msg);
-   g_free(dpip_tag2);
+   dFree(dialog_msg);
+   dFree(dpip_tag2);
 }
 /*-- EOD part */
 
    /* Start sending our answer */
    d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
-   sock_handler_write_str(sh, d_cmd, 0);
-   g_free(d_cmd);
+   sock_handler_write_str(sh, 0, d_cmd);
+   dFree(d_cmd);
 
    sock_handler_printf(sh, 0,
       "Content-type: text/html\n\n"
@@ -110,14 +115,14 @@ int main(void)
       "<pre>\n%s</pre>\n"
       "<br><small>(<b>dpip:</b> dpi protocol)</small><br><br><br>\n",
       esc_tag);
-   g_free(esc_tag);
+   dFree(esc_tag);
 
 
    /* Now something more interesting,
     * fork a command and show its feedback */
    if (cmd && url) {
-      child_cmd = g_strdup("date -R");
-      g_printerr("[%s]\n", child_cmd);
+      child_cmd = dStrdup("date -R");
+      MSG("[%s]\n", child_cmd);
 
       /* Fork, exec command, get its output and answer */
       if ((in_stream = popen(child_cmd, "r")) == NULL) {
@@ -130,22 +135,22 @@ int main(void)
 
       /* Read/Write */
       while ((n = fread (buf, 1, 4096, in_stream)) > 0) {
-         sock_handler_write(sh, buf, n, 0);
+         sock_handler_write(sh, 0, buf, n);
       }
 
       sock_handler_printf(sh, 0, "</pre>\n");
 
       if ((ret = pclose(in_stream)) != 0)
-         g_printerr("popen: [%d]\n", ret);
+         MSG("popen: [%d]\n", ret);
 
-      g_free(child_cmd);
+      dFree(child_cmd);
    }
 
    sock_handler_printf(sh, 1, "</body></html>\n");
 
-   g_free(cmd);
-   g_free(url);
-   g_free(dpip_tag);
+   dFree(cmd);
+   dFree(url);
+   dFree(dpip_tag);
 
    /* Finish the SockHandler */
    sock_handler_close(sh);
