@@ -25,7 +25,7 @@
  * the https dillo plugin with the OpenSSL project's "OpenSSL"
  * library, and distribute the linked executables, without including
  * the source code for OpenSSL in the source distribution. You must
- * obey the GNU General Public License, version 2, in all respects
+ * obey the GNU General Public License, version 3, in all respects
  * for all of the code used other than "OpenSSL".
  *
  */
@@ -64,12 +64,15 @@
 /*
  * Debugging macros
  */
+#define SILENT 1
 #define _MSG(...)
-#define MSG(...)  printf("[https dpi]: " __VA_ARGS__)
+#if SILENT
+ #define MSG(...)
+#else
+ #define MSG(...)  fprintf(stderr, "[https dpi]: " __VA_ARGS__)
+#endif
 
 
-#define ENABLE_SSL
-#undef ENABLE_SSL
 #ifdef ENABLE_SSL
 
 #include <openssl/ssl.h>
@@ -126,7 +129,8 @@ static void yes_ssl_support(void)
    SSL_CTX * ssl_context = NULL;
    SSL * ssl_connection = NULL;
 
-   char *dpip_tag = NULL, *cmd = NULL, *url = NULL, *http_query = NULL;
+   char *dpip_tag = NULL, *cmd = NULL, *url = NULL, *http_query = NULL,
+        *proxy_url = NULL, *proxy_connect = NULL;
    char buf[4096];
    int retval = 0;
    int network_socket = -1;
@@ -191,6 +195,9 @@ static void yes_ssl_support(void)
       /*Get the network address and command to be used*/
       dpip_tag = sock_handler_read(sh);
       cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
+      proxy_url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "proxy_url");
+      proxy_connect =
+                  a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "proxy_connect");
       url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
       http_query = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "query");
 
@@ -202,13 +209,66 @@ static void yes_ssl_support(void)
    }
 
    if (exit_error == 0){
-      network_socket = get_network_connection(url);
+      char *connect_url = proxy_url ? proxy_url : url;
+
+      network_socket = get_network_connection(connect_url);
       if (network_socket<0){
          MSG("Network socket create error\n");
          exit_error = 1;
       }
    }
 
+   if (exit_error == 0 && proxy_connect != NULL) {
+      ssize_t St;
+      const char *p = proxy_connect;
+      int writelen = strlen(proxy_connect);
+
+      while (writelen > 0) {
+         St = write(network_socket, p, writelen);
+         if (St < 0) {
+            /* Error */
+            if (errno != EINTR) {
+               MSG("Error writing to proxy.\n");
+               exit_error = 1;
+               break;
+            }
+         } else {
+            p += St;
+            writelen -= St;
+         }
+      }
+      if (exit_error == 0) {
+         const size_t buflen = 200;
+         char buf[buflen];
+         Dstr *reply = dStr_new("");
+
+         while (1) {
+            St = read(network_socket, buf, buflen);
+            if (St > 0) {
+               dStr_append_l(reply, buf, St);
+               if (strstr(reply->str, "\r\n\r\n")) {
+                  /* have whole reply header */
+                  if (reply->len >= 12 && reply->str[9] == '2') {
+                     /* e.g. "HTTP/1.1 200 Connection established[...]" */
+                     MSG("CONNECT through proxy succeeded.\n");
+                  } else {
+                     /* TODO: send reply body to dillo */
+                     exit_error = 1;
+                     MSG("CONNECT through proxy failed.\n");
+                  }
+                  break;
+               }
+            } else if (St < 0) {
+               if (errno != EINTR) {
+                  exit_error = 1;
+                  MSG("Error reading from proxy.\n");
+                  break;
+               }
+            }
+         }
+         dStr_free(reply, 1);
+      }
+   }
 
    if (exit_error == 0){
       /* Configure SSL to use network file descriptor */
@@ -260,6 +320,8 @@ static void yes_ssl_support(void)
    dFree(cmd);
    dFree(url);
    dFree(http_query);
+   dFree(proxy_url);
+   dFree(proxy_connect);
 
    if (network_socket != -1){
       close(network_socket);
@@ -288,12 +350,15 @@ static int get_network_connection(char * url)
    int s;
    int url_offset = 0;
    int portnum = 443;
-   unsigned int url_look_up_length = 0;
+   uint_t url_look_up_length = 0;
    char * url_look_up = NULL;
 
    /*Determine how much of url we chop off as unneeded*/
    if (dStrncasecmp(url, "https://", 8) == 0){
       url_offset = 8;
+   } else if (dStrncasecmp(url, "http://", 7) == 0) {
+      url_offset = 7;
+      portnum = 80;
    }
 
    /*Find end of URL*/
@@ -305,7 +370,7 @@ static int get_network_connection(char * url)
       /*Check for port number*/
       if (strchr(url+url_offset, ':') ==
           (url + url_offset + url_look_up_length)){
-         portnum = atoi(url + url_offset + url_look_up_length + 1);
+         portnum = strtol(url + url_offset + url_look_up_length + 1, NULL, 10);
       }
    } else {
       url_look_up = url + url_offset;
@@ -577,7 +642,7 @@ static int save_certificate_home(X509 * cert)
    char buf[4096];
 
    FILE * fp = NULL;
-   unsigned int i = 0;
+   uint_t i = 0;
    int retval = 1;
 
    /*Attempt to create .dillo/certs blindly - check later*/
@@ -586,7 +651,7 @@ static int save_certificate_home(X509 * cert)
    snprintf(buf,4096,"%s/.dillo/certs/", dGethomedir());
    mkdir(buf, 01777);
 
-   do{
+   do {
       snprintf(buf, 4096, "%s/.dillo/certs/%lx.%u",
                dGethomedir(), X509_subject_name_hash(cert), i);
 
@@ -609,7 +674,7 @@ static int save_certificate_home(X509 * cert)
       }
       i++;
       /*Don't loop too many times - just give up*/
-   } while( i < 1024 );
+   } while (i < 1024);
 
    return retval;
 }
@@ -655,23 +720,25 @@ static void no_ssl_support(void)
    sock_handler_printf(sh, 1,
       "Content-type: text/html\n\n"
       "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
-      "<html><body><pre>\n"
-      "<b>Hi!\n\n"
-      "  This is the https dpi that just got a request to send\n"
-      "  the following HTTP query:\n{</b>\n"
-      "<code>%s</code>\n"
-      "<b>}</b>\n\n"
-      "  <b>*** Dillo's prototype plugin for https support"
-      " is disabled now ***</b>\n\n"
-      "  If you want to test this <b>alpha</b> support code, just remove\n"
-      "  line 72 from dpi/https.c, recompile and reinstall.\n\n"
-      "  (beware that this https support is very limited now)\n\n"
+      "<html><head><title>SSL support is disabled</title></head>\n"
+      "<body>\n"
+      "<p>\n"
+      "  The https dpi was unable to send\n"
+      "  the following HTTP query:\n"
+      "  <blockquote><pre>%s</pre></blockquote>\n"
+      "  because Dillo's prototype plugin for https support"
+      "  is disabled.\n\n"
+      "<p>\n"
+      "  If you want to test this <b>alpha</b> support code,\n"
+      "  just reconfigure with <code>--enable-ssl</code>,\n"
+      "  recompile and reinstall.\n\n"
+      "  (Beware that this https support is very limited now)\n\n"
       "  To use https and SSL, you must have \n"
       "  the OpenSSL development libraries installed.  Check your\n"
       "  O/S distribution provider, or check out\n"
-      "  <a href=\"http://www.openssl.org\">www.openssl.org</a>\n\n"
-      "  --\n"
-      "</pre></body></html>\n",
+      "  <a href=\"http://www.openssl.org\">www.openssl.org</a>.\n\n"
+      "</p>\n\n"
+      "</body></html>\n",
       http_query
    );
    MSG("{ HTML content sent.}\n");
