@@ -3,7 +3,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -12,17 +12,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*! \file
  * Main functions to set-up dpi information and to initialise sockets
  */
 #include <errno.h>
-#include <glib.h>
-#include <dirent.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -33,16 +29,30 @@
 #include "dpi_service.h"
 #include "misc_new.h"
 
+#include "../dpip/dpip.h"
+
 #define QUEUE 5
 
 volatile sig_atomic_t caught_sigchld = 0;
 
+/*! Return the basename of a filename
+ */
+static char *get_basename(char *filename)
+{
+   char *p;
+
+   if (filename && (p = strrchr(filename, '/'))) {
+      filename = p + 1;
+   }
+   return filename;
+}
+
 /*! Close and remove the sockets in the
  * given dpi attribute list
  */
 void rm_dpi_sockets(struct dp *dpi_attr_list, int numdpis)
 {
-   gint i;
+   int i;
 
    for (i = 0; i < numdpis; i++) {
       a_Misc_close_fd(dpi_attr_list[i].socket);
@@ -82,11 +92,11 @@ void cleanup(char *socket_dir)
       return;
    }
    while ( (dir_entry = readdir(dir)) != NULL ) {
-      if ( dir_entry->d_name[0] == '.' )
+      if (dir_entry->d_name[0] == '.')
          continue;
-      sockpath = g_strconcat(socket_dir, "/", dir_entry->d_name, NULL);
+      sockpath = dStrconcat(socket_dir, "/", dir_entry->d_name, NULL);
       unlink(sockpath);
-      g_free(sockpath);
+      dFree(sockpath);
    }
    closedir(dir);
 }
@@ -97,15 +107,15 @@ void cleanup(char *socket_dir)
 void free_dpi_attr(struct dp *dpi_attr)
 {
    if (dpi_attr->id != NULL) {
-      g_free(dpi_attr->id);
+      dFree(dpi_attr->id);
       dpi_attr->id = NULL;
    }
    if (dpi_attr->path != NULL) {
-      g_free(dpi_attr->path);
+      dFree(dpi_attr->path);
       dpi_attr->path = NULL;
    }
    if (dpi_attr->sockpath != NULL) {
-      g_free(dpi_attr->sockpath);
+      dFree(dpi_attr->sockpath);
       dpi_attr->sockpath = NULL;
    }
 }
@@ -123,12 +133,26 @@ void free_plugin_list(struct dp **dpi_attr_list_ptr, int numdpis)
    for (i = 0; i < numdpis; i++)
       free_dpi_attr(dpi_attr_list + i);
 
-   g_free(dpi_attr_list);
+   dFree(dpi_attr_list);
    *dpi_attr_list_ptr = NULL;
 }
 
+/*! Free memory used by the services list
+ */
+void free_services_list(Dlist *s_list)
+{
+   int i = 0;
+   struct service *s;
+
+   for (i=0; i < dList_length(s_list) ; i++) {
+      s = dList_nth_data(s_list, i);
+      dFree(s->name);
+   }
+   dList_free(s_list);
+}
+
 /*! \todo
- * Remove terminator and est_terminator unless we really want to clean up 
+ * Remove terminator and est_terminator unless we really want to clean up
  * on abnormal exit.
  */
 #if 0
@@ -143,7 +167,7 @@ void terminator(int sig)
    _exit(0);
 }
 
-/*! Establish handler for termination signals 
+/*! Establish handler for termination signals
  * and register cleanup with atexit */
 void est_terminator(void)
 {
@@ -169,7 +193,7 @@ void est_terminator(void)
 
    if (atexit(cleanup) != 0) {
       ERRMSG("est_terminator", "atexit", 0);
-      fprintf(stderr, "Hey! atexit failed, how did that happen?\n");
+      MSG_ERR("Hey! atexit failed, how did that happen?\n");
       exit(1);
    }
 }
@@ -179,14 +203,14 @@ void est_terminator(void)
  * Currently there is only one file type associated with dpis.
  * More file types will be added as needed
  */
-enum file_type get_file_type(gchar *file_name)
+enum file_type get_file_type(char *file_name)
 {
-   gchar *dot = strrchr(file_name, '.');
+   char *dot = strrchr(file_name, '.');
 
    if (dot && !strcmp(dot, ".dpi"))
       return DPI_FILE;             /* Any filename ending in ".dpi" */
    else {
-      fprintf(stderr, "get_file_type: Unknown file type for %s\n", file_name);
+      MSG_ERR("get_file_type: Unknown file type for %s\n", file_name);
       return UNKNOWN_FILE;
    }
 }
@@ -210,13 +234,16 @@ int get_dpi_attr(char *dpi_dir, char *service, struct dp *dpi_attr)
    DIR *dir_stream;
    struct dirent *dir_entry = NULL;
 
-   service_dir = g_strconcat(dpi_dir, "/", service, NULL);
+   service_dir = dStrconcat(dpi_dir, "/", service, NULL);
    if (stat(service_dir, &statinfo) == -1) {
       ERRMSG("get_dpi_attr", "stat", errno);
-      fprintf(stderr, "file=%s\n", service_dir);
-   } else if ( (dir_stream = opendir(service_dir)) == NULL) {
+      MSG_ERR("file=%s\n", service_dir);
+   } else if ((dir_stream = opendir(service_dir)) == NULL) {
       ERRMSG("get_dpi_attr", "opendir", errno);
    } else {
+      /* Scan the directory looking for dpi files.
+       * (currently there's only the dpi program, but in the future
+       *  there may also be helper scripts.) */
       while ( (dir_entry = readdir(dir_stream)) != NULL) {
          if (dir_entry->d_name[0] == '.')
             continue;
@@ -225,28 +252,27 @@ int get_dpi_attr(char *dpi_dir, char *service, struct dp *dpi_attr)
          switch (ftype) {
             case DPI_FILE:
                dpi_attr->path =
-                  g_strconcat(service_dir, "/", dir_entry->d_name, NULL);
-               dpi_attr->id = g_strdup(service);
+                  dStrconcat(service_dir, "/", dir_entry->d_name, NULL);
+               dpi_attr->id = dStrdup(service);
                dpi_attr->sockpath = NULL;
                dpi_attr->pid = 1;
                if (strstr(dpi_attr->path, ".filter") != NULL)
                   dpi_attr->filter = 1;
                else
                   dpi_attr->filter = 0;
-               break;
-            case UNKNOWN_FILE:
+               retval = 0;
                break;
             default:
-               ERRMSG("Fatal error: get_dpi_attr", "get_file_type", 0);
-               fprintf(stderr, " - switch cases do not match ftype\n");
-               exit(1);
                break;
          }
       }
       closedir(dir_stream);
-      retval = 0;
+
+      if (retval != 0)
+         MSG_ERR("get_dpi_attr: No dpi plug-in in %s/%s\n",
+                 dpi_dir, service);
    }
-   g_free(service_dir);
+   dFree(service_dir);
    return retval;
 }
 
@@ -264,24 +290,23 @@ int register_service(struct dp *dpi_attr, char *service)
    char *user_dpi_dir, *dpidrc, *user_service_dir, *dir = NULL;
    int retval = -1;
 
-   /* g_get_home_dir makes it hard to test for mem leaks */
-   user_dpi_dir = g_strconcat(a_Misc_get_home(), "/", dotDILLO_DPI, NULL);
+   user_dpi_dir = dStrconcat(dGethomedir(), "/", dotDILLO_DPI, NULL);
    user_service_dir =
-       g_strconcat(a_Misc_get_home(), "/", dotDILLO_DPI, "/", service, NULL);
+       dStrconcat(dGethomedir(), "/", dotDILLO_DPI, "/", service, NULL);
 
-   dpidrc = g_strconcat(a_Misc_get_home(), "/", dotDILLO_DPIDRC, NULL);
-   if ( access(dpidrc, F_OK) == -1 ) {
-      if ( access(DPIDRC_SYS, F_OK) == -1 ) {
+   dpidrc = dStrconcat(dGethomedir(), "/", dotDILLO_DPIDRC, NULL);
+   if (access(dpidrc, F_OK) == -1) {
+      if (access(DPIDRC_SYS, F_OK) == -1) {
          ERRMSG("register_service", "Error ", 0);
-         fprintf(stderr, "\n - There is no %s or %s file\n", dpidrc,
+         MSG_ERR("\n - There is no %s or %s file\n", dpidrc,
                DPIDRC_SYS);
-         g_free(user_dpi_dir);
-         g_free(user_service_dir);
-         g_free(dpidrc);
+         dFree(user_dpi_dir);
+         dFree(user_service_dir);
+         dFree(dpidrc);
          return(-1);
       }
-      g_free(dpidrc);
-      dpidrc = g_strdup(DPIDRC_SYS);
+      dFree(dpidrc);
+      dpidrc = dStrdup(DPIDRC_SYS);
    }
 
    /* Check home dir for dpis */
@@ -294,21 +319,21 @@ int register_service(struct dp *dpi_attr, char *service)
             get_dpi_attr(dir, service, dpi_attr);
             retval = 0;
          } else {
-            ERRMSG("register_service", "get_dpi_attr failed\n", 0);
+            ERRMSG("register_service", "get_dpi_attr failed", 0);
          }
       } else {
-         ERRMSG("register_service", "dpi_dir: Error getting dpi dir.\n", 0);
+         ERRMSG("register_service", "dpi_dir: Error getting dpi dir.", 0);
       }
    }
-   g_free(user_dpi_dir);
-   g_free(user_service_dir);
-   g_free(dpidrc);
-   g_free(dir);
+   dFree(user_dpi_dir);
+   dFree(user_service_dir);
+   dFree(dpidrc);
+   dFree(dir);
    return (retval);
 }
 
 /*!
- * Create dpi directory for available 
+ * Create dpi directory for available
  * plugins and create plugin list.
  * \Return
  * \li Returns number of available plugins on success
@@ -318,39 +343,38 @@ int register_all(struct dp **attlist)
 {
    DIR *user_dir_stream, *sys_dir_stream;
    char *user_dpidir = NULL, *sys_dpidir = NULL, *dpidrc = NULL;
-   char *basename=NULL;
    struct dirent *user_dirent, *sys_dirent;
-   int j, not_in_user_list;
+   int st;
    int snum, usr_srv_num;
    size_t dp_sz = sizeof(struct dp);
 
    if (*attlist != NULL) {
-      ERRMSG("register_all", "attlist parameter should be NULL\n", 0);
+      ERRMSG("register_all", "attlist parameter should be NULL", 0);
       return -1;
    }
 
-   user_dpidir = g_strconcat(a_Misc_get_home(), "/", dotDILLO_DPI, NULL);
+   user_dpidir = dStrconcat(dGethomedir(), "/", dotDILLO_DPI, NULL);
    if (access(user_dpidir, F_OK) == -1) {
       /* no dpis in user's space */
-      g_free(user_dpidir);
+      dFree(user_dpidir);
       user_dpidir = NULL;
    }
-   dpidrc = g_strconcat(a_Misc_get_home(), "/", dotDILLO_DPIDRC, NULL);
+   dpidrc = dStrconcat(dGethomedir(), "/", dotDILLO_DPIDRC, NULL);
    if (access(dpidrc, F_OK) == -1) {
-      g_free(dpidrc);
-      dpidrc = g_strdup(DPIDRC_SYS);
+      dFree(dpidrc);
+      dpidrc = dStrdup(DPIDRC_SYS);
       if (access(dpidrc, F_OK) == -1) {
-         g_free(dpidrc);
+         dFree(dpidrc);
          dpidrc = NULL;
       }
    }
    if (!dpidrc || (sys_dpidir = get_dpi_dir(dpidrc)) == NULL)
       sys_dpidir = NULL;
-   g_free(dpidrc);
+   dFree(dpidrc);
 
    if (!user_dpidir && !sys_dpidir) {
       ERRMSG("register_all", "Fatal error ", 0);
-      fprintf(stderr, "\n - Can't find the directory for dpis.\n");
+      MSG_ERR("\n - Can't find the directory for dpis.\n");
       exit(1);
    }
 
@@ -360,8 +384,10 @@ int register_all(struct dp **attlist)
       while ((user_dirent = readdir(user_dir_stream)) != NULL) {
          if (user_dirent->d_name[0] == '.')
             continue;
-         *attlist = (struct dp *) g_realloc(*attlist, (snum + 1) * dp_sz);
-         get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum++]);
+         *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
+         st=get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum]);
+         if (st == 0)
+            snum++;
       }
       usr_srv_num = snum;
       closedir(user_dir_stream);
@@ -370,31 +396,131 @@ int register_all(struct dp **attlist)
       /* if system service is not in user list then add it */
       while ((sys_dirent = readdir(sys_dir_stream)) != NULL) {
          if (sys_dirent->d_name[0] == '.')
-           continue; 
-         not_in_user_list = 1;
-         for (j = 0; j < usr_srv_num; j++) {
-            basename = g_basename((*attlist)[j].path);
-            if (strcmp(sys_dirent->d_name, basename) == 0) {
-               not_in_user_list = 0;
-               break;
-            }
-         }
-         if (not_in_user_list) {
-            *attlist = (struct dp *) g_realloc(*attlist, (snum + 1) * dp_sz);
-            get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum++]);
-         }
+           continue;
+         *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
+         st=get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum]);
+         if (st == 0)
+            snum++;
       }
       closedir(sys_dir_stream);
    }
 
-   g_free(sys_dpidir);
-   g_free(user_dpidir);
+   dFree(sys_dpidir);
+   dFree(user_dpidir);
 
-   /* todo: do we consider snum == 0 an error?
+   /* TODO: do we consider snum == 0 an error?
     *       (if so, we should return -1 )       */
    return (snum);
 }
 
+/*
+ * Compare two struct service pointers
+ * This function is used for sorting services
+ */
+static int services_alpha_comp(const struct service *s1,
+                               const struct service *s2)
+{
+   return -strcmp(s1->name, s2->name);
+}
+
+/*! Add services reading a dpidrc file
+ * each non empty or commented line has the form
+ * service = path_relative_to_dpidir
+ * \Return:
+ * \li Returns number of available services on success
+ * \li -1 on failure
+ */
+int fill_services_list(struct dp *attlist, int numdpis, Dlist **services_list)
+{
+   FILE *dpidrc_stream;
+   char *p, *line = NULL, *service, *path;
+   int i, st;
+   struct service *s;
+   char *user_dpidir = NULL, *sys_dpidir = NULL, *dpidrc = NULL;
+
+   user_dpidir = dStrconcat(dGethomedir(), "/", dotDILLO_DPI, NULL);
+   if (access(user_dpidir, F_OK) == -1) {
+      /* no dpis in user's space */
+      dFree(user_dpidir);
+      user_dpidir = NULL;
+   }
+   dpidrc = dStrconcat(dGethomedir(), "/", dotDILLO_DPIDRC, NULL);
+   if (access(dpidrc, F_OK) == -1) {
+      dFree(dpidrc);
+      dpidrc = dStrdup(DPIDRC_SYS);
+      if (access(dpidrc, F_OK) == -1) {
+         dFree(dpidrc);
+         dpidrc = NULL;
+      }
+   }
+   if (!dpidrc || (sys_dpidir = get_dpi_dir(dpidrc)) == NULL)
+      sys_dpidir = NULL;
+
+   if (!user_dpidir && !sys_dpidir) {
+      ERRMSG("fill_services_list", "Fatal error ", 0);
+      MSG_ERR("\n - Can't find the directory for dpis.\n");
+      exit(1);
+   }
+
+   if ((dpidrc_stream = fopen(dpidrc, "r")) == NULL) {
+      ERRMSG("fill_services_list", "popen failed", errno);
+      dFree(dpidrc);
+      dFree(sys_dpidir);
+      dFree(user_dpidir);
+      return (-1);
+   }
+
+   if (*services_list != NULL) {
+      ERRMSG("fill_services_list", "services_list parameter is not NULL", 0);
+      return -1;
+   }
+   *services_list = dList_new(8);
+
+   /* dpidrc parser loop */
+   while ((line = dGetline(dpidrc_stream)) != NULL) {
+      st = dParser_parse_rc_line(&line, &service, &path);
+      if (st < 0) {
+         MSG_ERR("dpid: Syntax error in %s: service=\"%s\" path=\"%s\"\n",
+                 dpidrc, service, path);
+         continue;
+      } else if (st != 0) {
+         continue;
+      }
+
+      _MSG("dpid: service=%s, path=%s\n", service, path);
+
+      /* ignore dpi_dir silently */
+      if (strcmp(service, "dpi_dir") == 0)
+         continue;
+
+      s = dNew(struct service, 1);
+      /* init services list entry */
+      s->name = dStrdup(service);
+      s->dp_index = -1;
+
+      dList_append(*services_list, s);
+      /* search the dpi for a service by its path */
+      for (i = 0; i < numdpis; i++)
+          if ((p = strstr(attlist[i].path, path)) && *(p - 1) == '/' &&
+              strlen(p) == strlen(path))
+             break;
+      /* if the dpi exist bind service and dpi */
+      if (i < numdpis)
+         s->dp_index = i;
+
+      dFree(line);
+   }
+   fclose(dpidrc_stream);
+
+   dList_sort(*services_list, (dCompareFunc)services_alpha_comp);
+
+   dFree(dpidrc);
+   dFree(sys_dpidir);
+   dFree(user_dpidir);
+
+   return (dList_length(*services_list));
+}
+
 /*! Initialise the service request socket
  * \Return:
  * \li Number of sockets (1 == success)
@@ -407,7 +533,7 @@ int init_srs_socket(char *sockdir)
    size_t sun_path_len;
    socklen_t addr_sz;
 
-   srs_name = g_strconcat(sockdir, "/", SRS_NAME, NULL);
+   srs_name = dStrconcat(sockdir, "/", SRS_NAME, NULL);
    FD_ZERO(&sock_set);
 
    /* Initialise srs, service request socket on startup */
@@ -421,27 +547,27 @@ int init_srs_socket(char *sockdir)
    srs_sa.sun_family = AF_LOCAL;
 
    sun_path_len = sizeof(srs_sa.sun_path);
-   if ( strlen(srs_name) > sun_path_len) {
+   if (strlen(srs_name) > sun_path_len) {
       ERRMSG("init_srs_socket", "srs_name is too long", 0);
-      fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
-      fprintf(stderr, "\n - srs_name = %s\n", srs_name);
+      MSG_ERR("\n - it should be <= %lu chars", (ulong_t)sun_path_len);
+      MSG_ERR("\n - srs_name = %s\n", srs_name);
       return(retval);
-   } 
+   }
    strncpy(srs_sa.sun_path, srs_name, sun_path_len);
    addr_sz = (socklen_t) D_SUN_LEN(&srs_sa);
 
    if ((bind(srs, (struct sockaddr *) &srs_sa, addr_sz)) == -1) {
       if (errno == EADDRINUSE) {
          ERRMSG("init_srs_socket", "bind", errno);
-         fprintf(stderr, "srs_sa.sun_path = %s\n", srs_sa.sun_path);
+         MSG_ERR("srs_sa.sun_path = %s\n", srs_sa.sun_path);
          dpi_errno = dpid_srs_addrinuse;
       } else {
          ERRMSG("init_srs_socket", "bind", errno);
-         fprintf(stderr, "srs_sa.sun_path = %s\n", srs_sa.sun_path);
+         MSG_ERR("srs_sa.sun_path = %s\n", srs_sa.sun_path);
       }
    } else if (chmod(srs_sa.sun_path, S_IRUSR | S_IWUSR) == -1) {
       ERRMSG("init_srs_socket", "chmod", errno);
-      fprintf(stderr, "srs_sa.sun_path = %s\n", srs_sa.sun_path);
+      MSG_ERR("srs_sa.sun_path = %s\n", srs_sa.sun_path);
    } else if (listen(srs, QUEUE) == -1) {
       ERRMSG("init_srs_socket", "listen", errno);
    } else {
@@ -464,6 +590,7 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
    struct sockaddr_un sa;
    size_t sp_len;
    socklen_t addr_sz;
+   size_t sock_buflen = 8192;
 
    sp_len = sizeof(sa.sun_path);
    if ((s = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1) {
@@ -473,27 +600,32 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
    /* Set the socket FD to close on exec */
    fcntl(s, F_SETFD, FD_CLOEXEC | fcntl(s, F_GETFD));
 
+   /* set some buffering to increase the transfer's speed */
+   setsockopt(s, SOL_SOCKET, SO_SNDBUF,
+              &sock_buflen, (socklen_t)sizeof(sock_buflen));
+
    dpi_attr->socket = s;
    dpi_attr->sa.sun_family = AF_LOCAL;
-   dpi_nm = g_basename(dpi_attr->path);
+   dpi_nm = get_basename(dpi_attr->path);
 
-   dpi_attr->sockpath = g_strconcat(sockdir, "/", dpi_nm, NULL);
-   if ( strlen(dpi_attr->sockpath) > sp_len) {
+   dpi_attr->sockpath = dStrconcat(sockdir, "/", dpi_nm, "-XXXXXX", NULL);
+   a_Misc_mkfname(dpi_attr->sockpath);
+   if (strlen(dpi_attr->sockpath) > sp_len) {
       ERRMSG("init_all_dpi_sockets", "socket path is too long", 0);
-      fprintf(stderr, "\n - it should be <= %d chars", sp_len);
-      fprintf(stderr, "\n - socket path = %s\n", dpi_attr->sockpath);
+      MSG_ERR("\n - it should be <= %lu chars", (ulong_t)sp_len);
+      MSG_ERR("\n - socket path = %s\n", dpi_attr->sockpath);
       return(-1);
-   } 
+   }
    strncpy(dpi_attr->sa.sun_path, dpi_attr->sockpath, sp_len);
    addr_sz = (socklen_t) D_SUN_LEN(&dpi_attr->sa);
 
    if ((bind(s, (struct sockaddr *) &dpi_attr->sa, addr_sz)) == -1) {
       ERRMSG("init_all_dpi_sockets", "bind", errno);
-      fprintf(stderr, "%s\n", dpi_attr->sa.sun_path);
+      MSG_ERR("%s\n", dpi_attr->sa.sun_path);
       caught_error = 1;
    } else if (chmod(dpi_attr->sa.sun_path, S_IRUSR | S_IWUSR) == -1) {
       ERRMSG("init_all_dpi_sockets", "chmod", errno);
-      fprintf(stderr, "%s\n", dpi_attr->sa.sun_path);
+      MSG_ERR("%s\n", dpi_attr->sa.sun_path);
       caught_error = 1;
    } else if (listen(s, QUEUE) == -1) {
       ERRMSG("init_all_dpi_sockets", "listen", errno);
@@ -540,7 +672,8 @@ int init_all_dpi_sockets(struct dp *dpi_attr_list, char *sockdir)
  */
 void dpi_sigchld(int sig)
 {
-   caught_sigchld = 1;
+   if (sig == SIGCHLD)
+      caught_sigchld = 1;
 }
 
 /*! Called by main loop when caught_sigchld == 1 */
@@ -586,14 +719,16 @@ void est_dpi_sigchld(void)
  */
 void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
 {
-   int i, dpi_socket;
+   static char *DpiBye_cmd = NULL;
+   int i, dpi_socket, rc;
    struct sockaddr_un dpi_addr;
-   char *DpiBye_cmd = "<dpi cmd='DpiBye'>";
    struct sockaddr_un sa;
    size_t sun_path_len, addr_len;
 
+   if (!DpiBye_cmd)
+      DpiBye_cmd = a_Dpip_build_cmd("cmd=%s", "DpiBye");
+
    sun_path_len = sizeof(sa.sun_path);
-   addr_len = sizeof(dpi_addr);
 
    dpi_addr.sun_family = AF_LOCAL;
 
@@ -605,17 +740,22 @@ void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
       if ((dpi_socket = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1) {
          ERRMSG("stop_active_dpis", "socket", errno);
       }
-      if ( strlen(dpi_attr_list[i].sockpath) > sun_path_len) {
+      if (strlen(dpi_attr_list[i].sockpath) > sun_path_len) {
          ERRMSG("stop_active_dpis", "socket path is too long", 0);
-         fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
-         fprintf(stderr, "\n - socket path = %s\n", dpi_attr_list[i].sockpath);
-      } 
+         MSG_ERR("\n - it should be <= %lu chars",(ulong_t)sun_path_len);
+         MSG_ERR("\n - socket path = %s\n", dpi_attr_list[i].sockpath);
+      }
       strncpy(dpi_addr.sun_path, dpi_attr_list[i].sockpath, sun_path_len);
+      addr_len = D_SUN_LEN(&dpi_addr);
       if (connect(dpi_socket, (struct sockaddr *) &dpi_addr, addr_len) == -1) {
          ERRMSG("stop_active_dpis", "connect", errno);
-         fprintf(stderr, "%s\n", dpi_addr.sun_path);
+         MSG_ERR("%s\n", dpi_addr.sun_path);
+      }
+      rc = write(dpi_socket, DpiBye_cmd, strlen(DpiBye_cmd));
+      if (rc == -1) {
+         MSG("stop_active_dpis: Error on sending BYE command: %s\n",
+             dStrerror(errno));
       }
-      (void) write(dpi_socket, DpiBye_cmd, strlen(DpiBye_cmd));
       a_Misc_close_fd(dpi_socket);
    }
 }
@@ -636,7 +776,7 @@ void ignore_dpi_sockets(struct dp *dpi_attr_list, int numdpis)
 
 /*! Registers available dpis and stops active non-filter dpis.
  * Called when dpid receives
- * <dpi cmd='register' service='all'>
+ * cmd='register' service='all'
  * command
  * \Return
  * Number of available dpis
@@ -646,11 +786,14 @@ int register_all_cmd(char *sockdir)
    stop_active_dpis(dpi_attr_list, numdpis);
    rm_dpi_sockets(dpi_attr_list, numdpis);
    free_plugin_list(&dpi_attr_list, numdpis);
+   free_services_list(services_list);
+   services_list = NULL;
    numdpis = 0;
    numsocks = 1;                /* the srs socket */
    FD_ZERO(&sock_set);
    FD_SET(srs, &sock_set);
    numdpis = register_all(&dpi_attr_list);
+   fill_services_list(dpi_attr_list, numdpis, &services_list);
    numsocks = init_all_dpi_sockets(dpi_attr_list, sockdir);
    return (numdpis);
 }
@@ -662,42 +805,73 @@ int register_all_cmd(char *sockdir)
  */
 char *get_message(int sock, char *dpi_tag)
 {
-   char *msg;
+   char *msg, *d_cmd;
 
-   msg = a_Misc_get_attr_value(dpi_tag, (int) strlen(dpi_tag), "msg");
+   msg = a_Dpip_get_attr(dpi_tag, strlen(dpi_tag), "msg");
    if (msg == NULL) {
-      ERRMSG("get_message", "failed to parse msg\n", 0);
-      (void) CKD_WRITE(sock, tag_prserr);
+      ERRMSG("get_message", "failed to parse msg", 0);
+      d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                               "DpiError", "Failed to parse request");
+      (void) CKD_WRITE(sock, d_cmd);
+      dFree(d_cmd);
    }
    return (msg);
 }
 
+/*
+ * Compare a struct service pointer and a service name
+ * This function is used for searching services by name
+ */
+int service_match(const struct service *A, const char *B)
+{
+   int A_len, B_len, len;
+
+   A_len = strlen(A->name);
+   B_len = strlen(B);
+   len = MAX (A_len, B_len);
+
+   if (A->name[A_len - 1] == '*')
+      len = A_len - 1;
+
+   return(dStrncasecmp(A->name, B, len));
+}
+
 /*!
  * Send socket path that matches dpi_id to client
  */
-void send_sockpath(gint sock, gchar * dpi_tag, struct dp *dpi_attr_list)
+void send_sockpath(int sock, char *dpi_tag, struct dp *dpi_attr_list)
 {
-   gint i;
-   gchar *dpi_id;
-   char *req_rply = NULL, *dpi_cmd = "<dpi cmd='send_data' msg='%s'>";
+   int i;
+   char *dpi_id;
+   char *d_cmd;
+   struct service *serv;
 
-   g_return_if_fail((dpi_id = get_message(sock, dpi_tag)) != NULL);
+   dReturn_if_fail((dpi_id = get_message(sock, dpi_tag)) != NULL);
+
+   serv = dList_find_custom(services_list,dpi_id,(dCompareFunc)service_match);
+
+   if (serv == NULL || (i = serv->dp_index) == -1)
+      for (i = 0; i < numdpis; i++)
+         if (!strncmp(dpi_attr_list[i].id, dpi_id,
+        dpi_attr_list[i].id - strchr(dpi_attr_list[i].id, '.')))
+            break;
 
-   for (i = 0; i < numdpis; i++)
-      if (strstr(dpi_attr_list[i].id, dpi_id))
-         break;
    if (i < numdpis) {
       /* found */
       if (access(dpi_attr_list[i].path, F_OK) == -1) {
          ERRMSG("send_sockpath", "access", errno);
-         fprintf(stderr, " - %s\n", dpi_attr_list[i].sockpath);
-         (void) CKD_WRITE(sock, tag_dpi_unavailerr);
+         MSG_ERR(" - %s\n", dpi_attr_list[i].sockpath);
+         d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                                  "DpiError", "Plugin currently unavailable");
+         (void) CKD_WRITE(sock, d_cmd);
+         dFree(d_cmd);
       } else {
-         req_rply = g_strdup_printf(dpi_cmd, dpi_attr_list[i].sockpath);
-         (void) CKD_WRITE(sock, req_rply);
+         d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                                  "send_data", dpi_attr_list[i].sockpath);
+         (void) CKD_WRITE(sock, d_cmd);
+         dFree(d_cmd);
       }
    }
 
-   g_free(dpi_id);
-   g_free(req_rply);
+   dFree(dpi_id);
 }
