@@ -1,7 +1,17 @@
+/*
+ * File: dpid_common.c
+ *
+ * Copyright 2008 Jorge Arellano Cid <jcid@dillo.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <glib.h>
 #include "dpid_common.h"
 
 /*
@@ -9,9 +19,9 @@
  */
 void errmsg(char *caller, char *called, int errornum, char *file, int line)
 {
-   fprintf(stderr, "%s:%d: %s: %s : ", file, line, caller, called);
+   MSG_ERR("%s:%d: %s: %s\n", file, line, caller, called);
    if (errornum > 0)
-      fprintf(stderr, "%s\n", g_strerror(errornum));
+      MSG_ERR("%s\n", dStrerror(errornum));
 }
 
 /*! Selector function for scandir
@@ -38,7 +48,7 @@ ssize_t ckd_write(int fd, char *msg, char *file, int line)
       ret = write(fd, msg, strlen(msg));
    } while (ret == -1 && errno == EINTR);
    if (ret == -1) {
-      fprintf(stderr, "%s:%d: write: %s\n", file, line, g_strerror(errno));
+      MSG_ERR("%s:%d: write: %s\n", file, line, dStrerror(errno));
    }
    return (ret);
 }
