@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -23,19 +22,18 @@
 #include "fltkcore.hh"
 #include "fltkflatview.hh"
 #include "fltkcomplexbutton.hh"
+#include "../lout/msg.h"
 #include "../lout/misc.hh"
 
 #include <stdio.h>
 #include <fltk/Widget.h>
 #include <fltk/Group.h>
 #include <fltk/Input.h>
-#include <fltk/SecretInput.h>
 #include <fltk/TextEditor.h>
 #include <fltk/RadioButton.h>
 #include <fltk/CheckButton.h>
 #include <fltk/Choice.h>
 #include <fltk/Browser.h>
-#include <fltk/MultiBrowser.h>
 #include <fltk/Font.h>
 #include <fltk/draw.h>
 #include <fltk/Symbol.h>
@@ -63,6 +61,8 @@ FltkResource::FltkResource (FltkPlatform *platform)
    allocation.descent = 0;
 
    style = NULL;
+
+   enabled = true;
 }
 
 /**
@@ -91,7 +91,7 @@ FltkResource::~FltkResource ()
 
    }
    delete viewsAndWidgets;
-   if(style)
+   if (style)
       style->unref ();
 }
 
@@ -100,12 +100,14 @@ void FltkResource::attachView (FltkView *view)
    if (view->usesFltkWidgets ()) {
       ViewAndWidget *viewAndWidget = new ViewAndWidget();
       viewAndWidget->view = view;
-      
+
       viewAndWidget->widget = createNewWidget (&allocation);
       viewAndWidget->view->addFltkWidget (viewAndWidget->widget, &allocation);
       viewsAndWidgets->append (viewAndWidget);
       if (style)
          setWidgetStyle (viewAndWidget->widget, style);
+      if (! enabled)
+         viewAndWidget->widget->deactivate ();
    }
 }
 
@@ -120,7 +122,7 @@ void FltkResource::detachView (FltkView *view)
       }
    }
 
-   fprintf (stderr, "FltkResource::detachView: View not found.");
+   MSG_WARN("FltkResource::detachView: View not found.");
 }
 
 void FltkResource::sizeAllocate (core::Allocation *allocation)
@@ -152,7 +154,7 @@ void FltkResource::draw (core::View *view, core::Rectangle *area)
 
 void FltkResource::setStyle (core::style::Style *style)
 {
-   if(this->style)
+   if (this->style)
       this->style->unref ();
 
    this->style = style;
@@ -199,16 +201,52 @@ void FltkResource::setWidgetStyle (::fltk::Widget *widget,
       }
    }
 }
-   
+
+void FltkResource::setDisplayed(bool displayed)
+{
+   for (Iterator <ViewAndWidget> it = viewsAndWidgets->iterator ();
+        it.hasNext(); ) {
+      ViewAndWidget *viewAndWidget = it.getNext ();
+      if (displayed)
+         viewAndWidget->widget->show();
+      else
+         viewAndWidget->widget->hide();
+   }
+}
+
+bool FltkResource::displayed()
+{
+   bool ret;
+   Iterator <ViewAndWidget> it = viewsAndWidgets->iterator ();
+
+   if (it.hasNext()) {
+      ViewAndWidget *viewAndWidget = it.getNext ();
+      // visible() is not the same thing as being show()n exactly, but
+      // show()/hide() set it appropriately for our purposes.
+      ret = viewAndWidget->widget->visible();
+   } else {
+      ret = false;
+   }
+   return ret;
+}
+
 bool FltkResource::isEnabled ()
 {
-   /** \bug Not implemented. */
-   return true;
+   return enabled;
 }
 
 void FltkResource::setEnabled (bool enabled)
 {
-   /** \bug Not implemented. */
+   this->enabled = enabled;
+
+   for (Iterator <ViewAndWidget> it = viewsAndWidgets->iterator ();
+        it.hasNext(); ) {
+      ViewAndWidget *viewAndWidget = it.getNext ();
+      if (enabled)
+         viewAndWidget->widget->activate ();
+      else
+         viewAndWidget->widget->deactivate ();
+   }
 }
 
 // ----------------------------------------------------------------------
@@ -286,11 +324,47 @@ void FltkLabelButtonResource::sizeRequest (core::Requisition *requisition)
    }
 }
 
+/*
+ * Get FLTK state and translate to dw
+ *
+ * TODO: find a good home for this and the fltkviewbase.cc original.
+ */
+static core::ButtonState getDwButtonState ()
+{
+   int s1 = ::fltk::event_state ();
+   int s2 = (core::ButtonState)0;
+
+   if (s1 & ::fltk::SHIFT)   s2 |= core::SHIFT_MASK;
+   if (s1 & ::fltk::CTRL)    s2 |= core::CONTROL_MASK;
+   if (s1 & ::fltk::ALT)     s2 |= core::META_MASK;
+   if (s1 & ::fltk::BUTTON1) s2 |= core::BUTTON1_MASK;
+   if (s1 & ::fltk::BUTTON2) s2 |= core::BUTTON2_MASK;
+   if (s1 & ::fltk::BUTTON3) s2 |= core::BUTTON3_MASK;
+
+   return (core::ButtonState)s2;
+}
+
+static void setButtonEvent(dw::core::EventButton *event)
+{
+   event->xCanvas = ::fltk::event_x();
+   event->yCanvas = ::fltk::event_y();
+   event->state = getDwButtonState();
+   event->button = ::fltk::event_button();
+   event->numPressed = ::fltk::event_clicks() + 1;
+}
+
 void FltkLabelButtonResource::widgetCallback (::fltk::Widget *widget,
                                               void *data)
 {
-   if (widget->when () & ::fltk::WHEN_RELEASE)
-      ((FltkLabelButtonResource*)data)->emitActivate ();
+   if ((widget->when () & ::fltk::WHEN_RELEASE) &&
+       ((::fltk::event_key() == ::fltk::ReturnKey) ||
+        (::fltk::event_button() == ::fltk::LeftButton ||
+         ::fltk::event_button() == ::fltk::MiddleButton))) {
+      FltkLabelButtonResource *lbr = (FltkLabelButtonResource*) data;
+      dw::core::EventButton event;
+      setButtonEvent(&event);
+      lbr->emitClicked(&event);
+   }
 }
 
 const char *FltkLabelButtonResource::getLabel ()
@@ -336,12 +410,15 @@ void FltkComplexButtonResource::widgetCallback (::fltk::Widget *widget,
 {
    FltkComplexButtonResource *res = (FltkComplexButtonResource*)data;
 
-   /* would be best not to send click pos. if the image could not be loaded */
-   if (::fltk::event() == ::fltk::RELEASE &&
-       ::fltk::event_button() == ::fltk::LeftButton) {
+   if (widget->when() == ::fltk::WHEN_RELEASE &&
+       ((::fltk::event_key() == ::fltk::ReturnKey) ||
+        (::fltk::event_button() == ::fltk::LeftButton ||
+         ::fltk::event_button() == ::fltk::MiddleButton))) {
       res->click_x = ::fltk::event_x();
       res->click_y = ::fltk::event_y();
-      res->emitActivate ();
+      dw::core::EventButton event;
+      setButtonEvent(&event);
+      res->emitClicked(&event);
    } else {
       ((FltkViewBase*)res->lastFlatView)->handle(::fltk::event());
    }
@@ -377,9 +454,8 @@ void FltkComplexButtonResource::detachView (FltkView *view)
       }
    }
 
-   fprintf (stderr,
-            "FltkComplexButtonResourceResource::detachView: View not "
-            "found.\n");
+   MSG_WARN("FltkComplexButtonResourceResource::detachView: "
+            "View not found.\n");
 }
 
 void FltkComplexButtonResource::sizeAllocate (core::Allocation *allocation)
@@ -430,7 +506,7 @@ int FltkComplexButtonResource::reliefYThickness ()
    if (!relief)
       button->box(::fltk::FLAT_BOX);
 
-   FltkFlatView *flatView = 
+   FltkFlatView *flatView =
       new FltkFlatView (allocation->x + reliefXThickness (),
                         allocation->y + reliefYThickness (),
                         allocation->width - 2 * reliefXThickness (),
@@ -448,12 +524,13 @@ int FltkComplexButtonResource::reliefYThickness ()
 // ----------------------------------------------------------------------
 
 FltkEntryResource::FltkEntryResource (FltkPlatform *platform, int maxLength,
-                                      bool password):  
+                                      bool password, const char *label):
    FltkSpecificResource <dw::core::ui::EntryResource> (platform)
 {
    this->maxLength = maxLength;
    this->password = password;
-   
+   this->label = label ? strdup(label) : NULL;
+
    initText = NULL;
    editable = false;
 
@@ -464,21 +541,25 @@ FltkEntryResource::~FltkEntryResource ()
 {
    if (initText)
       delete initText;
+   if (label)
+      delete label;
 }
 
 ::fltk::Widget *FltkEntryResource::createNewWidget (core::Allocation
                                                     *allocation)
 {
    ::fltk::Input *input =
-        password ?
-        new ::fltk::SecretInput (allocation->x, allocation->y,
-                                 allocation->width,
-                                 allocation->ascent + allocation->descent) :
         new ::fltk::Input (allocation->x, allocation->y, allocation->width,
                            allocation->ascent + allocation->descent);
+   if (password)
+      input->type(::fltk::Input::SECRET);
    input->callback (widgetCallback, this);
    input->when (::fltk::WHEN_ENTER_KEY_ALWAYS);
 
+   if (label) {
+      input->label(label);
+      input->set_flag(::fltk::ALIGN_INSIDE_LEFT);
+   }
    if (viewsAndWidgets->isEmpty ()) {
       // First widget created, attach the set text.
       if (initText)
@@ -490,20 +571,26 @@ FltkEntryResource::~FltkEntryResource ()
    return input;
 }
 
+void FltkEntryResource::setDisplayed(bool displayed)
+{
+   FltkResource::setDisplayed(displayed);
+   queueResize(true);
+}
+
 void FltkEntryResource::sizeRequest (core::Requisition *requisition)
 {
-   if (style) {
+   if (displayed() && style) {
       FltkFont *font = (FltkFont*)style->font;
       ::fltk::setfont(font->font,font->size);
       requisition->width =
-         (int)::fltk::getwidth ("M", 1)
+         (int)::fltk::getwidth ("n", 1)
          * (maxLength == UNLIMITED_MAX_LENGTH ? 10 : maxLength)
          + 2 * RELIEF_X_THICKNESS;
       requisition->ascent = font->ascent + RELIEF_Y_THICKNESS;
       requisition->descent = font->descent + RELIEF_Y_THICKNESS;
    } else {
-      requisition->width = 1;
-      requisition->ascent = 1;
+      requisition->width = 0;
+      requisition->ascent = 0;
       requisition->descent = 0;
    }
 }
@@ -517,7 +604,7 @@ void FltkEntryResource::widgetCallback (::fltk::Widget *widget,
     * The Back or Forward, buttons, or the first click on a rendered
     * page. BUG: this must be investigated and reported to FLTK2 team
     */
-   printf ("when = %d\n", widget->when ());
+   MSG("when = %d\n", widget->when ());
    if ((widget->when () & ::fltk::WHEN_ENTER_KEY_ALWAYS) &&
        (::fltk::event_key() == ::fltk::ReturnKey))
       ((FltkEntryResource*)data)->emitActivate ();
@@ -557,7 +644,7 @@ void FltkEntryResource::setEditable (bool editable)
 // ----------------------------------------------------------------------
 
 FltkMultiLineTextResource::FltkMultiLineTextResource (FltkPlatform *platform,
-                                                      int cols, int rows): 
+                                                      int cols, int rows):
    FltkSpecificResource <dw::core::ui::MultiLineTextResource> (platform)
 {
    buffer = new ::fltk::TextBuffer;
@@ -568,11 +655,11 @@ FltkMultiLineTextResource::FltkMultiLineTextResource (FltkPlatform *platform,
 
    // Check values. Upper bound check is left to the caller.
    if (numCols < 1) {
-      fprintf (stderr, "WARNING: numCols = %d is set to 1.\n", numCols);
+      MSG_WARN("numCols = %d is set to 1.\n", numCols);
       numCols = 1;
    }
    if (numRows < 1) {
-      fprintf (stderr, "WARNING: numRows = %d is set to 1.\n", numRows);
+      MSG_WARN("numRows = %d is set to 1.\n", numRows);
       numRows = 1;
    }
 
@@ -607,13 +694,13 @@ void FltkMultiLineTextResource::sizeRequest (core::Requisition *requisition)
       FltkFont *font = (FltkFont*)style->font;
       ::fltk::setfont(font->font,font->size);
       requisition->width =
-         (int)::fltk::getwidth ("X", 1) * numCols +
+         (int)::fltk::getwidth ("n", 1) * numCols +
          2 * RELIEF_X_THICKNESS;
       requisition->ascent =
-         font->ascent + RELIEF_Y_THICKNESS;
+         RELIEF_Y_THICKNESS + font->ascent +
+         (font->ascent + font->descent) * (numRows - 1);
       requisition->descent =
          font->descent +
-         (font->ascent + font->descent) * (numRows - 1) +
          RELIEF_Y_THICKNESS;
    } else {
       requisition->width = 1;
@@ -647,7 +734,7 @@ void FltkMultiLineTextResource::setEditable (bool editable)
 template <class I>
 FltkToggleButtonResource<I>::FltkToggleButtonResource (FltkPlatform *platform,
                                                        bool activated):
-   FltkSpecificResource <I> (platform)  
+   FltkSpecificResource <I> (platform)
 {
    initActivated = activated;
 }
@@ -736,7 +823,7 @@ FltkCheckButtonResource::~FltkCheckButtonResource ()
 
 
 ::fltk::Button *FltkCheckButtonResource::createNewButton (core::Allocation
-                                                          *allocation)   
+                                                          *allocation)
 {
    ::fltk::CheckButton *cb =
       new ::fltk::CheckButton (allocation->x, allocation->y, allocation->width,
@@ -775,7 +862,7 @@ FltkRadioButtonResource::Group::~Group ()
 {
    delete list;
 }
-      
+
 void FltkRadioButtonResource::Group::connect (FltkRadioButtonResource
                                               *radioButtonResource)
 {
@@ -836,7 +923,7 @@ void FltkRadioButtonResource::buttonClicked ()
 }
 
 ::fltk::Button *FltkRadioButtonResource::createNewButton (core::Allocation
-                                                          *allocation)   
+                                                          *allocation)
 {
    /*
     * Groups of fltk::RadioButton must be added to one fltk::Group, which is
@@ -1025,7 +1112,7 @@ template <class I> void FltkSelectionResource<I>::addItem (const char *str,
       }
    }
 }
-   
+
 template <class I> void FltkSelectionResource<I>::pushGroup (const char *name,
                                                              bool enabled)
 {
@@ -1038,7 +1125,7 @@ template <class I> void FltkSelectionResource<I>::pushGroup (const char *name,
       ::fltk::ItemGroup *group = item->createNewGroupWidget ();
       widgetStack->stack->getTop()->getTypedValue()->add (group);
       widgetStack->stack->push (new TypedPointer < ::fltk::Menu> (group));
-      if(!enabled)
+      if (!enabled)
          group->deactivate ();
    }
 }
@@ -1123,7 +1210,7 @@ void FltkOptionMenuResource::sizeRequest (core::Requisition *requisition)
       requisition->width = 1;
       requisition->ascent = 1;
       requisition->descent = 0;
-   } 
+   }
 }
 
 void FltkOptionMenuResource::addItem (const char *str,
@@ -1146,10 +1233,12 @@ bool FltkOptionMenuResource::isSelected (int index)
 
 FltkListResource::FltkListResource (FltkPlatform *platform,
                                     core::ui::ListResource::SelectionMode
-                                    selectionMode):
+                                    selectionMode, int rowCount):
    FltkSelectionResource <dw::core::ui::ListResource> (platform),
    itemsSelected(8)
 {
+   mode = selectionMode;
+   showRows = rowCount;
    init (platform);
 }
 
@@ -1161,9 +1250,10 @@ FltkListResource::~FltkListResource ()
 ::fltk::Menu *FltkListResource::createNewMenu (core::Allocation *allocation)
 {
    ::fltk::Menu *menu =
-      new ::fltk::MultiBrowser (allocation->x, allocation->y,
-                                allocation->width,
-                                allocation->ascent + allocation->descent);
+      new ::fltk::Browser (allocation->x, allocation->y, allocation->width,
+                           allocation->ascent + allocation->descent);
+   if (mode == SELECTION_MULTIPLE)
+      menu->type(::fltk::Browser::MULTI);
    menu->set_flag (::fltk::RAW_LABEL);
    menu->callback(widgetCallback,this);
    menu->when(::fltk::WHEN_CHANGED);
@@ -1173,10 +1263,25 @@ FltkListResource::~FltkListResource ()
 void FltkListResource::widgetCallback (::fltk::Widget *widget, void *data)
 {
    ::fltk::Widget *fltkItem = ((::fltk::Menu *) widget)->item ();
-   int index = (long) (fltkItem->user_data ());
+   int index = -1;
+   if (fltkItem)
+      index = (long) (fltkItem->user_data ());
    if (index > -1) {
-      bool selected = fltkItem->selected ();
-      ((FltkListResource *) data)->itemsSelected.set (index, selected);
+      /* A MultiBrowser will trigger a callback for each item that is
+       * selected and each item that is deselected, but a "plain"
+       * Browser will only trigger the callback for the newly selected item
+       * (for which selected() is false, incidentally).
+       */
+      FltkListResource *res = (FltkListResource *) data;
+      if (res->mode == SELECTION_MULTIPLE) {
+         bool selected = fltkItem->selected ();
+         res->itemsSelected.set (index, selected);
+      } else {
+         int size = res->itemsSelected.size();
+         for (int i = 0; i < size; i++)
+            res->itemsSelected.set (i, false);
+         res->itemsSelected.set (index, true);
+      }
    }
 }
 
@@ -1195,18 +1300,24 @@ void FltkListResource::sizeRequest (core::Requisition *requisition)
    if (style) {
       FltkFont *font = (FltkFont*)style->font;
       ::fltk::setfont(font->font,font->size);
-      int maxStringWidth = getMaxStringWidth ();
-      requisition->ascent = font->ascent + RELIEF_Y_THICKNESS;
-      requisition->descent =
-         (getNumberOfItems () - 1) * (font->ascent) + font->descent;
-      requisition->width = maxStringWidth
-         + (font->ascent + font->descent) * 4 / 5
-         + 2 * RELIEF_X_THICKNESS;
+      int rows = getNumberOfItems();
+      if (showRows < rows) {
+         rows = showRows;
+      }
+      /*
+       * The widget sometimes shows scrollbars when they are not required.
+       * The following values try to keep any scrollbars from obscuring
+       * options, at the cost of showing too much whitespace at times.
+       */
+      requisition->width = getMaxStringWidth() + 24;
+      requisition->ascent = font->ascent + 2 +
+                            (rows - 1) * (font->ascent + font->descent + 1);
+      requisition->descent = font->descent + 3;
    } else {
       requisition->width = 1;
       requisition->ascent = 1;
       requisition->descent = 0;
-   } 
+   }
 }
 
 bool FltkListResource::isSelected (int index)
