@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
 
 #include "core.hh"
 
+#include "../lout/msg.h"
 #include "../lout/debug.hh"
 #include "../lout/misc.hh"
 
@@ -38,8 +38,8 @@ void Layout::Receiver::canvasSizeChanged (int width, int ascent, int descent)
 // ----------------------------------------------------------------------
 
 bool Layout::Emitter::emitToReceiver (lout::signal::Receiver *receiver,
-                                      int signalNo,
-                                      int argc, Object **argv)
+                                      int signalNo, int argc,
+                                      lout::object::Object **argv)
 {
    Receiver *layoutReceiver = (Receiver*)receiver;
 
@@ -54,7 +54,7 @@ bool Layout::Emitter::emitToReceiver (lout::signal::Receiver *receiver,
       misc::assertNotReached ();
    }
 
-   return false;      
+   return false;
 }
 
 void Layout::Emitter::emitCanvasSizeChanged (int width,
@@ -87,9 +87,11 @@ Layout::Layout (Platform *platform)
    cursor = style::CURSOR_DEFAULT;
 
    canvasWidth = canvasAscent = canvasDescent = 0;
-   
+
    usesViewport = false;
    scrollX = scrollY = 0;
+   viewportWidth = viewportHeight = 0;
+   hScrollbarThickness = vScrollbarThickness = 0;
 
    requestedAnchor = NULL;
    scrollIdleId = -1;
@@ -128,7 +130,7 @@ Layout::~Layout ()
 void Layout::addWidget (Widget *widget)
 {
    if (topLevel) {
-      fprintf (stderr, "widget already set\n");
+      MSG_WARN("widget already set\n");
       return;
    }
 
@@ -247,7 +249,7 @@ void Layout::detachView (View *view)
 }
 
 /**
- * \brief Scrolls all viewports, so that the region [x, y, width, height] 
+ * \brief Scrolls all viewports, so that the region [x, y, width, height]
  *    is seen, according to hpos and vpos.
  */
 void Layout::scrollTo (HPosition hpos, VPosition vpos,
@@ -261,8 +263,8 @@ void Layout::scrollTo0 (HPosition hpos, VPosition vpos,
                         bool scrollingInterrupted)
 {
    if (usesViewport) {
-      //printf ("scrollTo (%d, %d, %s)\n",
-      //        x, y, scrollingInterrupted ? "true" : "false");
+      _MSG("scrollTo (%d, %d, %s)\n",
+           x, y, scrollingInterrupted ? "true" : "false");
 
       scrollTargetHpos = hpos;
       scrollTargetVpos = vpos;
@@ -270,12 +272,12 @@ void Layout::scrollTo0 (HPosition hpos, VPosition vpos,
       scrollTargetY = y;
       scrollTargetWidth = width;
       scrollTargetHeight = height;
-      
+
       if (scrollIdleId == -1) {
          scrollIdleId = platform->addIdle (&Layout::scrollIdle);
          scrollIdleNotInterrupted = true;
       }
-      
+
       scrollIdleNotInterrupted =
          scrollIdleNotInterrupted || !scrollingInterrupted;
    }
@@ -353,7 +355,7 @@ void Layout::adjustScrollPos ()
       canvasAscent + canvasDescent - (viewportHeight - hScrollbarThickness));
    scrollY = misc::max (scrollY, 0);
 
- //printf("adjustScrollPos: scrollX=%d scrollY=%d\n", scrollX, scrollY);
+   _MSG("adjustScrollPos: scrollX=%d scrollY=%d\n", scrollX, scrollY);
 }
 
 bool Layout::calcScrollInto (int requestedValue, int requestedSize,
@@ -413,7 +415,7 @@ void Layout::draw (View *view, Rectangle *area)
  */
 void Layout::setAnchor (const char *anchor)
 {
-   //printf ("setAnchor (%s)\n", anchor);
+   _MSG("setAnchor (%s)\n", anchor);
 
    if (requestedAnchor)
       delete requestedAnchor;
@@ -449,7 +451,7 @@ char *Layout::addAnchor (Widget *widget, const char* name, int y)
 
 void Layout::changeAnchor (Widget *widget, char* name, int y)
 {
-   String key (name);   
+   String key (name);
    Anchor *anchor = anchorsTable->get (&key);
    assert (anchor);
    assert (anchor->widget == widget);
@@ -487,7 +489,7 @@ void Layout::setCursor (style::Cursor cursor)
 {
    if (cursor != this->cursor) {
       this->cursor = cursor;
-      
+
       for (typed::Iterator <View> it = views->iterator (); it.hasNext (); ) {
          View *view = it.getNext ();
          view->setCursor (cursor);
@@ -523,7 +525,7 @@ void Layout::updateBgColor ()
 void Layout::resizeIdle ()
 {
    //static int calls = 0;
-   //printf(" Layout::resizeIdle calls = %d\n", ++calls);
+   //MSG(" Layout::resizeIdle calls = %d\n", ++calls);
 
    while (resizeIdleId != -1) {
       // Reset already here, since in this function, queueResize() may be
@@ -563,7 +565,7 @@ void Layout::resizeIdle ()
             (canvasAscent + canvasDescent > viewportHeight) ?
             vScrollbarThickness : 0;
 
-            if (!canvasHeightGreater &&         
+            if (!canvasHeightGreater &&
                canvasAscent + canvasDescent
                > viewportHeight - actualHScrollbarThickness) {
                canvasHeightGreater = true;
@@ -591,7 +593,7 @@ void Layout::resizeIdle ()
 }
 
 void Layout::setSizeHints ()
-{ 
+{
    if (topLevel) {
       topLevel->setWidth (viewportWidth
                           - (canvasHeightGreater ? vScrollbarThickness : 0));
@@ -622,15 +624,15 @@ void Layout::queueDrawExcept (int x, int y, int width, int height,
 
    if (x == ex && y == ey && width == ewidth && height == eheight)
       return;
-  
+
    // queueDraw() the four rectangles within rectangle (x, y, width, height)
    // around rectangle (ex, ey, ewidth, eheight).
    // Some or all of these may be empty.
 
-   // upper left corner of the intersection rectangle 
+   // upper left corner of the intersection rectangle
    int ix1 = misc::max (x, ex);
    int iy1 = misc::max (y, ey);
-   // lower right corner of the intersection rectangle 
+   // lower right corner of the intersection rectangle
    int ix2 = misc::min (x + width, ex + ewidth);
    int iy2 = misc::min (y + height, ey + eheight);
 
@@ -658,10 +660,10 @@ void Layout::queueResize ()
 
 bool Layout::buttonEvent (ButtonEventType type, View *view, int numPressed,
                           int x, int y, ButtonState state, int button)
-   
+
 {
    EventButton event;
-   
+
    moveToWidgetAtPoint (x, y, state);
 
    event.xCanvas = x;
@@ -669,7 +671,7 @@ bool Layout::buttonEvent (ButtonEventType type, View *view, int numPressed,
    event.state = state;
    event.button = button;
    event.numPressed = numPressed;
-   
+
    return processMouseEvent (&event, type, true);
 }
 
@@ -682,13 +684,13 @@ bool Layout::buttonEvent (ButtonEventType type, View *view, int numPressed,
 bool Layout::motionNotify (View *view,  int x, int y, ButtonState state)
 {
    EventButton event;
-   
+
    moveToWidgetAtPoint (x, y, state);
 
    event.xCanvas = x;
    event.yCanvas = y;
    event.state = state;
-   
+
    return processMouseEvent (&event, MOTION_NOTIFY, true);
 }
 
@@ -705,7 +707,7 @@ void Layout::enterNotify (View *view, int x, int y, ButtonState state)
    lastWidget = widgetAtPoint;
    moveToWidgetAtPoint (x, y, state);
 
-   if(widgetAtPoint) {
+   if (widgetAtPoint) {
       event.state = state;
       event.lastWidget = lastWidget;
       event.currentWidget = widgetAtPoint;
@@ -726,7 +728,7 @@ void Layout::leaveNotify (View *view, ButtonState state)
    lastWidget = widgetAtPoint;
    moveOutOfView (state);
 
-   if(lastWidget) {
+   if (lastWidget) {
       event.state = state;
       event.lastWidget = lastWidget;
       event.currentWidget = widgetAtPoint;
@@ -739,8 +741,8 @@ void Layout::leaveNotify (View *view, ButtonState state)
  */
 Widget *Layout::getWidgetAtPoint (int x, int y)
 {
-   //_MSG ("------------------------------------------------------------\n");
-   //_MSG ("widget at (%d, %d)\n", x, y);
+   _MSG ("------------------------------------------------------------\n");
+   _MSG ("widget at (%d, %d)\n", x, y);
    if (topLevel)
       return topLevel->getWidgetAtPoint (x, y, 0);
    else
@@ -760,11 +762,11 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
    EventCrossing crossingEvent;
 
    if (newWidgetAtPoint != widgetAtPoint) {
-      // The mouse pointer has been moved into another widget. 
+      // The mouse pointer has been moved into another widget.
       if (newWidgetAtPoint && widgetAtPoint)
          ancestor =
             newWidgetAtPoint->getNearestCommonAncestor (widgetAtPoint);
-      else if(newWidgetAtPoint)
+      else if (newWidgetAtPoint)
          ancestor = newWidgetAtPoint->getTopLevel ();
       else
          ancestor = widgetAtPoint->getTopLevel ();
@@ -776,7 +778,7 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
          for (w = widgetAtPoint; w != ancestor; w = w->getParent ())
             trackLen++;
       trackLen++; // for the ancestor
-      if(newWidgetAtPoint)
+      if (newWidgetAtPoint)
          // second part
          for (w = newWidgetAtPoint; w != ancestor; w = w->getParent ())
             trackLen++;
@@ -788,7 +790,7 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
          for (w = widgetAtPoint; w != ancestor; w = w->getParent ())
             track[i++] = w;
       track[i++] = ancestor;
-      if(newWidgetAtPoint) {
+      if (newWidgetAtPoint) {
          /* second part */
          i = trackLen - 1;
          for (w = newWidgetAtPoint; w != ancestor; w = w->getParent ())
@@ -800,13 +802,13 @@ void Layout::moveToWidget (Widget *newWidgetAtPoint, ButtonState state)
          crossingEvent.state = state;
          crossingEvent.currentWidget = widgetAtPoint; // ???
          crossingEvent.lastWidget = widgetAtPoint; // ???
-         
+
          if (i != 0)
             track[i]->enterNotify (&crossingEvent);
          if (i != trackLen - 1)
             track[i]->leaveNotify (&crossingEvent);
       }
-      
+
       delete[] track;
 
       widgetAtPoint = newWidgetAtPoint;
@@ -826,7 +828,7 @@ bool Layout::processMouseEvent (MousePositionEvent *event,
    Widget *widget;
 
    for (widget = widgetAtPoint; widget; widget = widget->getParent ()) {
-      if(!mayBeSuppressed || widget->isButtonSensitive ()) {
+      if (!mayBeSuppressed || widget->isButtonSensitive ()) {
          event->xWidget = event->xCanvas - widget->getAllocation()->x;
          event->yWidget = event->yCanvas - widget->getAllocation()->y;
 
@@ -859,12 +861,12 @@ void Layout::scrollPosChanged (View *view, int x, int y)
    if (x != scrollX || y != scrollY) {
       scrollX = x;
       scrollY = y;
-      
+
       // Tell all views about the scrolling position, except the caller.
       for (container::typed::Iterator <View> it = views->iterator ();
            it.hasNext (); ) {
          View *thisView = it.getNext();
-         if(view != thisView && thisView->usesViewport ())
+         if (view != thisView && thisView->usesViewport ())
             thisView->scrollTo (scrollX, scrollY);
       }
 
@@ -879,12 +881,12 @@ void Layout::scrollPosChanged (View *view, int x, int y)
  */
 void Layout::viewportSizeChanged (View *view, int width, int height)
 {
-   //printf("Layout::viewportSizeChanged w=%d h=%d new_w=%d new_h=%d\n",
-   //       viewportWidth, viewportHeight, width, height);
+   _MSG("Layout::viewportSizeChanged w=%d h=%d new_w=%d new_h=%d\n",
+        viewportWidth, viewportHeight, width, height);
 
    /* If the width has become higher, we test again, whether the vertical
     * scrollbar (so to speak) can be hidden again. */
-   if(usesViewport && width > viewportWidth)
+   if (usesViewport && width > viewportWidth)
       canvasHeightGreater = false;
 
    /* if size changes, redraw this view.
@@ -906,7 +908,7 @@ void Layout::viewportSizeChanged (View *view, int width, int height)
    for (container::typed::Iterator <View> it = views->iterator ();
         it.hasNext (); ) {
       View *thisView = it.getNext();
-      if(view != thisView && thisView->usesViewport ())
+      if (view != thisView && thisView->usesViewport ())
          thisView->setViewportSize (viewportWidth, viewportHeight,
                                     actualHScrollbarThickness,
                                     actualVScrollbarThickness);
