@@ -14,13 +14,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
 
 #include "textblock.hh"
+#include "../lout/msg.h"
 #include "../lout/misc.hh"
 
 #include <stdio.h>
@@ -260,8 +260,8 @@ void Textblock::getExtremesImpl (core::Extremes *extremes)
                   extremes->minWidth = wordExtremes.minWidth;
             }
 
-            //printf("parMax = %d, wordMaxWidth=%d, prevWordSpace=%d\n",
-            //       parMax, wordExtremes.maxWidth, prevWordSpace);
+            _MSG("parMax = %d, wordMaxWidth=%d, prevWordSpace=%d\n",
+                 parMax, wordExtremes.maxWidth, prevWordSpace);
             if (word->content.type != core::Content::BREAK)
                parMax += prevWordSpace;
             parMax += wordExtremes.maxWidth;
@@ -370,9 +370,9 @@ void Textblock::sizeAllocateImpl (core::Allocation *allocation)
                // + word->content.widget->getStyle()->margin.bottom;
 
             oldChildAllocation = word->content.widget->getAllocation();
-   
-            if (childAllocation.x != oldChildAllocation->x || 
-               childAllocation.y != oldChildAllocation->y || 
+
+            if (childAllocation.x != oldChildAllocation->x ||
+               childAllocation.y != oldChildAllocation->y ||
                childAllocation.width != oldChildAllocation->width) {
                /* The child widget has changed its position or its width
                 * so we need to redraw from this line onwards.
@@ -392,7 +392,7 @@ void Textblock::sizeAllocateImpl (core::Allocation *allocation)
                 * end up redrawing the whole screen over and over.
                 * The drawing of the child content is left to the child itself.
                 */
-               int childChangedY = 
+               int childChangedY =
                   misc::min(childAllocation.y - allocation->y +
                      childAllocation.ascent + childAllocation.descent,
                      oldChildAllocation->y - this->allocation.y +
@@ -1077,8 +1077,8 @@ void Textblock::calcWidgetSize (core::Widget *widget, core::Requisition *size)
       widget->setAscent (availAscent);
       widget->setDescent (availDescent);
       widget->sizeRequest (size);
-      size->ascent -= widget->getStyle()->margin.top;
-      size->descent -= widget->getStyle()->margin.bottom;
+//      size->ascent -= widget->getStyle()->margin.top;
+//      size->descent -= widget->getStyle()->margin.bottom;
    } else {
       /* TODO: Use margin.{top|bottom} here, like above.
        * (No harm for the next future.) */
@@ -1201,6 +1201,31 @@ void Textblock::rewrap ()
    //DBG_MSG_END (page);
 }
 
+/*
+ * Draw the decorations on a word.
+ */
+void Textblock::decorateText(core::View *view, core::style::Style *style,
+                             core::style::Color::Shading shading,
+                             int x, int yBase, int width)
+{
+   int y;
+
+   if (style->textDecoration & core::style::TEXT_DECORATION_UNDERLINE) {
+      y = yBase + 1;
+      view->drawLine (style->color, shading, x, y, x + width - 1, y);
+   }
+   if (style->textDecoration & core::style::TEXT_DECORATION_OVERLINE) {
+      y = yBase - style->font->ascent + 1;
+      view->drawLine (style->color, shading, x, y, x + width - 1, y);
+   }
+   if (style->textDecoration & core::style::TEXT_DECORATION_LINE_THROUGH) {
+      int height = 1 + style->font->xHeight / 10;
+
+      y = yBase + (style->font->descent - style->font->ascent) / 2;
+      view->drawRectangle (style->color, shading, true, x, y, width, height);
+   }
+}
+
 /*
  * Paint a line
  * - x and y are toplevel dw coordinates (Question: what Dw? Changed. Test!)
@@ -1209,187 +1234,189 @@ void Textblock::rewrap ()
  */
 void Textblock::drawLine (Line *line, core::View *view, core::Rectangle *area)
 {
-   Word *word;
-   int wordIndex;
-   int xWidget, yWidget, xWorld, yWorld, yWorldBase;
-   int startHL, widthHL;
-   int wordLen;
-   int diff, effHLStart, effHLEnd, layer;
-   core::Widget *child;
-   core::Rectangle childArea;
-   core::style::Color *color, *thisBgColor, *wordBgColor;
+   int xWidget = lineXOffsetWidget(line);
+   int yWidgetBase = lineYOffsetWidget (line) + line->ascent;
+   int yWorldBase = allocation.y + yWidgetBase;
 
    /* Here's an idea on how to optimize this routine to minimize the number
-    * of calls to gdk_draw_string:
+    * of drawing calls:
     *
     * Copy the text from the words into a buffer, adding a new word
     * only if: the attributes match, and the spacing is either zero or
     * equal to the width of ' '. In the latter case, copy a " " into
     * the buffer. Then draw the buffer. */
 
-   thisBgColor = getBgColor ();
+   core::style::Color *thisBgColor = getBgColor ();
 
-   xWidget = lineXOffsetWidget(line);
-   xWorld = allocation.x + xWidget;
-   yWidget = lineYOffsetWidget (line);
-   yWorld = allocation.y + yWidget;
-   yWorldBase = yWorld + line->ascent;
-
-   for (wordIndex = line->firstWord; wordIndex < line->lastWord;
+   for (int wordIndex = line->firstWord; wordIndex < line->lastWord;
         wordIndex++) {
-      word = words->getRef(wordIndex);
-      diff = 0;
-      color = word->style->color;
+      int xWorld = allocation.x + xWidget;
+      Word *word = words->getRef(wordIndex);
 
       //DBG_OBJ_ARRSET_NUM (page, "words.%d.<i>drawn at</i>.x", wordIndex,
       //                    xWidget);
       //DBG_OBJ_ARRSET_NUM (page, "words.%d.<i>drawn at</i>.y", wordIndex,
-      //                    yWidget);
+      //                    yWidgetBase - line->ascent);
 
       switch (word->content.type) {
       case core::Content::TEXT:
-         if (word->style->backgroundColor)
-            wordBgColor = word->style->backgroundColor;
-         else
-            wordBgColor = thisBgColor;
+      {
+         int yWordShift = 0, ySpaceShift = 0;
 
-         /* Adjust the text baseline if the word is <SUP>-ed or <SUB>-ed. */
-         if (word->style->valign == core::style::VALIGN_SUB)
-            diff = word->size.ascent / 2;
-         else if (word->style->valign == core::style::VALIGN_SUPER)
-            diff -= word->size.ascent / 3;
+         if (word->size.width > 0) {
+            /* Draw word */
 
-         /* Draw background (color, image), when given. */
-         if (word->style->hasBackground () && word->size.width > 0)
-            drawBox (view, word->style, area,
-                     xWidget, yWidget + line->ascent - word->size.ascent,
-                     word->size.width, word->size.ascent + word->size.descent,
-                     false);
+            /* Adjust the text baseline if the word is <SUP>-ed or <SUB>-ed. */
+            if (word->style->valign == core::style::VALIGN_SUB)
+               yWordShift = word->style->font->ascent / 3;
+            else if (word->style->valign == core::style::VALIGN_SUPER) {
+               yWordShift -= word->style->font->ascent / 2;
+            }
+            /* Draw background (color, image), when given. */
+            if (word->style->hasBackground ())
+               drawBox (
+                  view, word->style, area,
+                  xWidget,
+                  yWidgetBase + yWordShift - word->style->font->ascent,
+                  word->size.width,
+                  word->style->font->ascent + word->style->font->descent,
+                  false);
 
-         /* Draw space background (color, image), when given. */
-         if (word->spaceStyle->hasBackground () && word->effSpace > 0)
-            drawBox (view, word->spaceStyle, area,
-                     xWidget + word->size.width,
-                     yWidget + line->ascent - word->size.ascent,
-                     word->effSpace, word->size.ascent + word->size.descent,
-                     false);
-         view->drawText (word->style->font, color,
-                         core::style::Color::SHADING_NORMAL,
-                         xWorld, yWorldBase + diff,
-                         word->content.text, strlen (word->content.text));
-
-         /* underline */
-         if (word->style->textDecoration &
-             core::style::TEXT_DECORATION_UNDERLINE)
-            view->drawLine (color, core::style::Color::SHADING_NORMAL,
-                            xWorld, yWorldBase + 1 + diff,
-                            xWorld + word->size.width - 1,
-                            yWorldBase + 1 + diff);
-         if (wordIndex + 1 < line->lastWord &&
-             (word->spaceStyle->textDecoration
-              & core::style::TEXT_DECORATION_UNDERLINE))
-            view->drawLine (word->spaceStyle->color,
+            view->drawText (word->style->font, word->style->color,
                             core::style::Color::SHADING_NORMAL,
-                            xWorld + word->size.width,
-                            yWorldBase + 1 + diff,
-                            xWorld + word->size.width + word->effSpace - 1,
-                            yWorldBase + 1 + diff);
+                            xWorld, yWorldBase + yWordShift,
+                            word->content.text, strlen (word->content.text));
 
-         /* strike-through */
-         if (word->style->textDecoration
-             & core::style::TEXT_DECORATION_LINE_THROUGH)
-            view->drawLine (color, core::style::Color::SHADING_NORMAL,
-                            xWorld,
-                            yWorldBase - word->size.ascent / 2 + diff,
-                            xWorld + word->size.width - 1,
-                            yWorldBase - word->size.ascent / 2 + diff);
-         if (wordIndex + 1 < line->lastWord &&
-             (word->spaceStyle->textDecoration
-              & core::style::TEXT_DECORATION_LINE_THROUGH))
-            view->drawLine (word->spaceStyle->color,
+            if (word->style->textDecoration)
+               decorateText(view, word->style,
+                            core::style::Color::SHADING_NORMAL, xWorld,
+                            yWorldBase + yWordShift, word->size.width);
+         }
+
+         if (word->effSpace > 0 && (wordIndex < line->lastWord - 1)) {
+            /* Draw space unless at end of line */
+
+            /* Adjust the space baseline if the word is <SUP>-ed or <SUB>-ed */
+            if (word->spaceStyle->valign == core::style::VALIGN_SUB)
+               ySpaceShift = word->spaceStyle->font->ascent / 3;
+            else if (word->spaceStyle->valign == core::style::VALIGN_SUPER) {
+               ySpaceShift -= word->spaceStyle->font->ascent / 2;
+            }
+
+            /* Draw space background (color, image), when given. */
+            if (word->spaceStyle->hasBackground ())
+               drawBox (
+                  view, word->spaceStyle, area,
+                  xWidget + word->size.width,
+                  yWidgetBase + ySpaceShift - word->spaceStyle->font->ascent,
+                  word->effSpace,
+                  word->spaceStyle->font->ascent +
+                     word->spaceStyle->font->descent,
+                  false);
+
+            if (word->spaceStyle->textDecoration)
+               decorateText(view, word->spaceStyle,
                             core::style::Color::SHADING_NORMAL,
-                            xWorld + word->size.width,
-                            yWorldBase - word->size.ascent / 2 + diff,
-                            xWorld + word->size.width + word->effSpace - 1,
-                            yWorldBase - word->size.ascent / 2 + diff);
+                            xWorld + word->size.width, yWorldBase +ySpaceShift,
+                            word->effSpace);
+         }
 
-         for (layer = 0; layer < core::HIGHLIGHT_NUM_LAYERS; layer++) {
+         for (int layer = 0; layer < core::HIGHLIGHT_NUM_LAYERS; layer++) {
             if (hlStart[layer].index <= wordIndex &&
                 hlEnd[layer].index >= wordIndex) {
+               const int wordLen = strlen (word->content.text);
+               int xStart, width;
+               int firstCharIdx = 0;
+               int lastCharIdx = wordLen;
 
-               wordLen = strlen (word->content.text);
-               effHLEnd = misc::min (wordLen, hlEnd[layer].nChar);
-               effHLStart = 0;
                if (wordIndex == hlStart[layer].index)
-                  effHLStart = misc::min (hlStart[layer].nChar, wordLen);
+                  firstCharIdx = misc::min (hlStart[layer].nChar, wordLen);
 
-               effHLEnd = wordLen;
                if (wordIndex == hlEnd[layer].index)
-                  effHLEnd = misc::min (hlEnd[layer].nChar, wordLen);
+                  lastCharIdx = misc::min (hlEnd[layer].nChar, wordLen);
 
-               startHL = xWorld + layout->textWidth (word->style->font,
-                                                     word->content.text,
-                                                     effHLStart);
-               widthHL =
-                  layout->textWidth (word->style->font,
-                                     word->content.text + effHLStart,
-                                     effHLEnd - effHLStart);
+               xStart = xWorld + layout->textWidth (word->style->font,
+                                                    word->content.text,
+                                                    firstCharIdx);
+               width = layout->textWidth (word->style->font,
+                                           word->content.text + firstCharIdx,
+                                           lastCharIdx - firstCharIdx);
 
-               // If the space after this word highlighted, and this word
-               // is not the last one in this line, highlight also the
-               // space.
-               /** \todo This should also be done with spaces after non-text
-                *        words, but this is not yet defined very well. */
-               if (wordIndex < hlEnd[layer].index &&
-                   wordIndex < words->size () &&
-                   wordIndex != line->lastWord - 1)
-                  widthHL += word->effSpace;
+               if (width > 0) {
+                  /* Highlight text */
+                  core::style::Color *wordBgColor;
 
+                  if (!(wordBgColor =  word->style->backgroundColor))
+                     wordBgColor = thisBgColor;
 
-               if (widthHL != 0) {
                   /* Draw background for highlighted text. */
-                  view->drawRectangle (wordBgColor,
-                                       core::style::Color::SHADING_INVERSE,
-                                       true, startHL,
-                                       yWorldBase - word->size.ascent,
-                                       widthHL,
-                                       word->size.ascent + word->size.descent);
+                  view->drawRectangle (
+                     wordBgColor,
+                     core::style::Color::SHADING_INVERSE,
+                     true, xStart,
+                     yWorldBase + yWordShift - word->style->font->ascent,
+                     width,
+                     word->style->font->ascent + word->style->font->descent);
 
                   /* Highlight the text. */
-                  view->drawText (word->style->font,
-                                  color, core::style::Color::SHADING_INVERSE,
-                                  startHL, yWorldBase + diff,
-                                  word->content.text + effHLStart,
-                                  effHLEnd - effHLStart);
+                  view->drawText (word->style->font, word->style->color,
+                                  core::style::Color::SHADING_INVERSE,
+                                  xStart, yWorldBase + yWordShift,
+                                  word->content.text + firstCharIdx,
+                                  lastCharIdx - firstCharIdx);
 
-                  /* underline and strike-through */
-                  if (word->style->textDecoration
-                      & core::style::TEXT_DECORATION_UNDERLINE)
-                     view->drawLine (color,
-                                     core::style::Color::SHADING_INVERSE,
-                                     startHL, yWorldBase + 1 + diff,
-                                     startHL + widthHL - 1,
-                                     yWorldBase + 1 + diff);
-                  if (word->style->textDecoration
-                      & core::style::TEXT_DECORATION_LINE_THROUGH)
-                     view->drawLine (color,
-                                     core::style::Color::SHADING_INVERSE,
-                                     startHL,
-                                     yWorldBase - word->size.ascent / 2 + diff,
-                                     startHL + widthHL - 1,
-                                     yWorldBase - word->size.ascent / 2
-                                     + diff);
+                  if (word->style->textDecoration)
+                     decorateText(view, word->style,
+                                  core::style::Color::SHADING_INVERSE, xStart,
+                                  yWorldBase + yWordShift, width);
+               }
+               xStart += width;
+
+               // Show highlighted space unless this is the last word on the
+               // line.
+               /** \todo This should also be done with spaces after non-text
+                *        words, but this is not yet defined very well. */
+
+               width = 0;
+               if (wordIndex < hlEnd[layer].index &&
+                   wordIndex < line->lastWord - 1)
+                  width = word->effSpace;
+
+               if (width > 0) {
+                  /* Highlight space */
+                  core::style::Color *spaceBgColor;
+
+                  if (!(spaceBgColor = word->spaceStyle->backgroundColor))
+                     spaceBgColor = thisBgColor;
+
+                  view->drawRectangle (
+                     spaceBgColor, core::style::Color::SHADING_INVERSE,
+                     true, xStart,
+                     yWorldBase + ySpaceShift - word->spaceStyle->font->ascent,
+                     width,
+                     word->spaceStyle->font->ascent +
+                        word->spaceStyle->font->descent);
+
+                  if (word->spaceStyle->textDecoration) {
+                     decorateText(view, word->spaceStyle,
+                                  core::style::Color::SHADING_INVERSE,
+                                  xStart, yWorldBase + ySpaceShift,
+                                  width);
+                  }
                }
             }
          }
          break;
-
+      }
       case core::Content::WIDGET:
-         child = word->content.widget;
+      {
+         core::Widget *child = word->content.widget;
+         core::Rectangle childArea;
+
          if (child->intersects (area, &childArea))
             child->draw (view, &childArea);
          break;
-
+      }
       case core::Content::ANCHOR: case core::Content::BREAK:
          /* nothing - an anchor/break isn't seen */
          /*
@@ -1417,11 +1444,9 @@ void Textblock::drawLine (Line *line, core::View *view, core::Rectangle *area)
          break;
 
       default:
-         fprintf (stderr,  "BUG!!! at (%d, %d).\n", xWorld, yWorldBase + diff);
+         MSG_ERR("at (%d, %d).\n", xWorld, yWorldBase);
          break;
       }
-
-      xWorld += word->size.width + word->effSpace;
       xWidget += word->size.width + word->effSpace;
    }
 }
@@ -1581,9 +1606,9 @@ void Textblock::calcTextSize (const char *text, core::style::Style *style,
     * potentially the line's height.
     */
    if (style->valign == core::style::VALIGN_SUB)
-      size->descent += (size->ascent / 2);
+      size->descent += (size->ascent / 3);
    else if (style->valign == core::style::VALIGN_SUPER)
-      size->ascent += (size->ascent / 3);
+      size->ascent += (size->ascent / 2);
 }
 
 
@@ -1658,7 +1683,7 @@ bool Textblock::addAnchor (const char *name, core::style::Style *style)
    char *copy;
    int y;
 
-   // Since an anchor does not take any space, it is safe to call 
+   // Since an anchor does not take any space, it is safe to call
    // addAnchor already here.
    if (wasAllocated ()) {
       if (lines->size () == 0)
@@ -1825,9 +1850,9 @@ void Textblock::addLinebreak (core::style::Style *style)
 
 /**
  * \brief Search recursively through widget.
- * 
+ *
  * This is an optimized version of the general
- * dw::core::Widget::getWidgetAtPoint method. 
+ * dw::core::Widget::getWidgetAtPoint method.
  */
 core::Widget  *Textblock::getWidgetAtPoint(int x, int y, int level)
 {
@@ -1921,14 +1946,14 @@ void Textblock::changeLinkColor (int link, int newColor)
             case core::Content::TEXT:
             {  core::style::Style *old_style = word->style;
                styleAttrs = *old_style;
-               styleAttrs.color = core::style::Color::createSimple (layout,
-                                                                    newColor);
+               styleAttrs.color = core::style::Color::create (layout,
+                                                              newColor);
                word->style = core::style::Style::create (layout, &styleAttrs);
                old_style->unref();
                old_style = word->spaceStyle;
                styleAttrs = *old_style;
-               styleAttrs.color = core::style::Color::createSimple (layout,
-                                                                    newColor);
+               styleAttrs.color = core::style::Color::create (layout,
+                                                              newColor);
                word->spaceStyle =
                                core::style::Style::create(layout, &styleAttrs);
                old_style->unref();
@@ -1937,10 +1962,10 @@ void Textblock::changeLinkColor (int link, int newColor)
             case core::Content::WIDGET:
             {  core::Widget *widget = word->content.widget;
                styleAttrs = *widget->getStyle();
-               styleAttrs.color = core::style::Color::createSimple (layout,
-                                                                    newColor);
+               styleAttrs.color = core::style::Color::create (layout,
+                                                              newColor);
                styleAttrs.setBorderColor(
-                           core::style::Color::createShaded(layout, newColor));
+                           core::style::Color::create (layout, newColor));
                widget->setStyle(
                              core::style::Style::create (layout, &styleAttrs));
                break;
@@ -1997,14 +2022,14 @@ int Textblock::TextblockIterator::compareTo(misc::Comparable *other)
 {
    return index - ((TextblockIterator*)other)->index;
 }
- 
+
 bool Textblock::TextblockIterator::next ()
 {
    Textblock *textblock = (Textblock*)getWidget();
 
    if (content.type == core::Content::END)
       return false;
-  
+
    do {
       index++;
       if (index >= textblock->words->size ()) {
@@ -2023,7 +2048,7 @@ bool Textblock::TextblockIterator::prev ()
 
    if (content.type == core::Content::START)
       return false;
-  
+
    do {
       index--;
       if (index < 0) {
