@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
 
 #include "core.hh"
 
+#include "../lout/msg.h"
 #include "../lout/debug.hh"
 
 using namespace lout::object;
@@ -54,8 +54,8 @@ void Widget::EventReceiver::leaveNotify (Widget *widget, EventCrossing *event)
 
 
 bool Widget::EventEmitter::emitToReceiver (lout::signal::Receiver *receiver,
-                                           int signalNo,
-                                           int argc, Object **argv)
+                                           int signalNo, int argc,
+                                           lout::object::Object **argv)
 {
    EventReceiver *eventReceiver = (EventReceiver*)receiver;
 
@@ -152,8 +152,8 @@ bool Widget::LinkReceiver::click (Widget *widget, int link, int img,
 
 
 bool Widget::LinkEmitter::emitToReceiver (lout::signal::Receiver *receiver,
-                                          int signalNo,
-                                          int argc, Object **argv)
+                                          int signalNo, int argc,
+                                          lout::object::Object **argv)
 {
    LinkReceiver *linkReceiver = (LinkReceiver*)receiver;
 
@@ -197,7 +197,7 @@ bool Widget::LinkEmitter::emitToReceiver (lout::signal::Receiver *receiver,
    return false;
 }
 
-bool Widget::LinkEmitter::emitEnter (Widget *widget, int link, int img, 
+bool Widget::LinkEmitter::emitEnter (Widget *widget, int link, int img,
                                      int x, int y)
 {
    Integer ilink (link), iimg (img), ix (x), iy (y);
@@ -237,7 +237,7 @@ int Widget::CLASS_ID = -1;
 Widget::Widget ()
 {
    registerName ("dw::core::Widget", &CLASS_ID);
-   
+
    flags = (Flags)(NEEDS_RESIZE | EXTREMES_CHANGED | HAS_CONTENTS);
    parent = NULL;
    layout = NULL;
@@ -315,7 +315,7 @@ void Widget::queueDrawArea (int x, int y, int width, int height)
 {
    /** \todo Maybe only the intersection? */
    layout->queueDraw (x + allocation.x, y + allocation.y, width, height);
- //printf("Widget::queueDrawArea x=%d y=%d w=%d h=%d\n", x, y, width, height);
+   _MSG("Widget::queueDrawArea x=%d y=%d w=%d h=%d\n", x, y, width, height);
 }
 
 /**
@@ -495,7 +495,7 @@ void Widget::leaveNotify (EventCrossing *event)
  * The old style is automatically unreferred, the new is referred. If this
  * call causes the widget to change its size, dw::core::Widget::queueResize
  * is called.
- */ 
+ */
 void Widget::setStyle (style::Style *style)
 {
    bool sizeChanged;
@@ -547,7 +547,7 @@ style::Color *Widget::getBgColor ()
       widget = widget->parent;
    }
 
-   fprintf (stderr, "No background color found!\n");
+   MSG_WARN("No background color found!\n");
    return NULL;
 
 }
@@ -692,10 +692,10 @@ Widget *Widget::getNearestCommonAncestor (Widget *otherWidget)
    /* Search upwards. */
    while (widget1 != widget2) {
       if (widget1->parent == NULL) {
-         fprintf (stderr, "widgets in different trees\n");
+         MSG_WARN("widgets in different trees\n");
          return NULL;
       }
-      
+
       widget1 = widget1->parent;
       widget2 = widget2->parent;
    }
@@ -706,7 +706,7 @@ Widget *Widget::getNearestCommonAncestor (Widget *otherWidget)
 
 /**
  * \brief Search recursively through widget.
- * 
+ *
  * Used by dw::core::Layout:getWidgetAtPoint.
  */
 Widget *Widget::getWidgetAtPoint (int x, int y, int level)
@@ -738,7 +738,7 @@ Widget *Widget::getWidgetAtPoint (int x, int y, int level)
                                                                     level + 1);
 
       it->unref ();
-      
+
       if (childAtPoint)
          return childAtPoint;
       else
