@@ -14,7 +14,7 @@
 /*
  * HTML text for startup screen
  */
-const char *AboutSplash=
+const char *const AboutSplash=
 "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
 "<html>\n"
 "<head>\n"
@@ -75,7 +75,7 @@ const char *AboutSplash=
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
-"     <a href='http://cvs.auriga.wearlab.de/cgi-bin/cvsweb.cgi/dillo2/ChangeLog?rev=HEAD;cvsroot=dillo'>\n"
+"     <a href='http://hg.dillo.org/dillo/file/tip/ChangeLog'>\n"
 "     ChangeLog</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
@@ -191,7 +191,7 @@ const char *AboutSplash=
 "     <tr><td>&nbsp;&nbsp;\n"
 "     <td><a href='http://www.violence.de'>Peace&amp;Violence</a>\n"
 "     <tr><td>&nbsp;&nbsp;\n"
-"     <td><a href='http://www.gnu.org/philosophy/right-to-read.html'>"
+"     <td><a href='http://www.gnu.org/philosophy/right-to-read.html'>\n"
 "      Right to Read</a>\n"
 "     </table>\n"
 "     </table>\n"
@@ -217,7 +217,8 @@ const char *AboutSplash=
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
 "  <p>\n"
-"  Dillo is Free Software in the terms of the GPL3.\n"
+"  Dillo is Free Software under the terms of version 3 of the\n"
+"  <a href='http://www.gnu.org/licenses/gpl.html'>GPL</a>.\n"
 "  This means you have four basic freedoms:\n"
 "  <ul>\n"
 "   <li>Freedom to use the program any way you see fit.\n"
@@ -225,10 +226,9 @@ const char *AboutSplash=
 "   <li>Freedom to make backup copies.\n"
 "   <li>Freedom to redistribute it.\n"
 "  </ul>\n"
-"  The <a href='http://www.gnu.org/licenses/gpl.html'>GPL3</a>\n"
-"  is the legal mechanism that gives you these freedoms.\n"
-"  It also protects them from being taken away: any derivative work\n"
-"  based on the program must be under the GPL3.<br>\n"
+"  The GPL is the legal mechanism that gives you these freedoms.\n"
+"  It also protects you from having them taken away: any derivative work\n"
+"  based on the program must be under GPLv3 as well.<br>\n"
 "  </table>\n"
 "</table>\n"
 "</table>\n"
@@ -240,25 +240,19 @@ const char *AboutSplash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  October 14, 2008\n"
+"  July 3, 2009\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"This is a rewrite of dillo, using FLTK2, that comes\n"
-"with lots of improvements and fixes.\n"
+"This release of Dillo improves image size checking. It builds upon version\n"
+"2.1, which brought significant improvements such as basic CSS support and\n"
+"configurable keybindings.\n"
 "<p>\n"
-"Our users will surely enjoy this new release as it will give them\n"
-"the  same  things they're accustomed plus tabbed browsing,\n"
-"antialiasing, different\n"
-"character  sets,  accepting  compressed pages, control over image\n"
-"loading, smaller footprint, fewer dependencies, better table\n"
-"rendering, bugfixes, improved GUI, ... In brief, a better dillo.\n"
-"<p>\n"
-"Remember that dillo project uses a release model where every new\n"
-"browser shall be better than the former.\n"
+"Remember that the dillo project uses a release model where every new\n"
+"version shall be better than the last.\n"
 "<EM>Keep up with the latest one!</EM>\n"
 "  </table>\n"
 "</table>\n"
@@ -272,69 +266,72 @@ const char *AboutSplash=
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>ChangeLog highlights</h4>\n"
 "  (Extracted from the\n"
-"  <a href='http://cvs.auriga.wearlab.de/cgi-bin/cvsweb.cgi/dillo2/ChangeLog?rev=HEAD;cvsroot=dillo'>full\n"
+"  <a href='http://hg.dillo.org/dillo/file/tip/ChangeLog'>full\n"
 "  ChangeLog</a>)\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
+"2.1.1:<br>\n"
 "<ul>\n"
-"<li>Ported Dillo from GTK1 to FLTK2."
-"<li>Ported a susbstantial part of the code from C to C++ (FLTK2 is in C++)."
-"<li>Wrote a new library: Dlib. With 'Dlib' Dillo doesn't need glib anymore."
-"<li>Ported all the code to Dlib."
-"<li>Made Dillo's UI Control Panel resizable on-the-fly."
-"<li>Implemented a new, simpler, dillorc parser."
-"<li>Reimplemented the Concomitant Callback chains into a uniform scheme!"
-"<li>Removed threads from IO. Now it only uses select-based watches."
-"<li>Simplified http.c by reusing the new non-blocking writes in IO."
-"<li>Implemented Stop button to not only stop rendering but also networking."
-"<li>Bound Ctrl+Space to toggle fullscreen mode."
-"<li>Added a http_referer preference. See details in dillorc."
-"<li>CCC: added reentrancy control to the OpEnd and OpAbort operations."
-"<li>CCC: enhanced the debug function and implemented OpAbort for dpi."
-"<li>Hooked a decoder for text/plain with charset."
-"<li>Forbid dpi GET and POST from non dpi-generated urls."
-"<li>Implemented tabbed browsing."
-"<li>Added a image-loading toggle button to the UI."
-"<li>Added line numbers and enabled wrapping in the 'View Source' window."
-"<li>Added HTTP-1.1's chunked transfer support!"
-"<li>Made the stop button sensitive when loading an image."
-"<li>Added support for 'charset' in the HTTP header field for Content-Type."
-"<li>Added support for 'charset' in the META element."
-"<li>Added the multipart/form-data encoding method to form submission."
-"<li>Made zlib a configure requirement, and cleaned up configure.in."
-"<li>Enabled the file dpi to look inside gzipped files."
-"<li>Added code for optional image loading (nice interface)!"
-"<li>Fixed data guesser to detect ASCII, LATIN1, UTF8, KOI8-R, CP-1251 as"
-" text."
-"<li>Fixed void to int conversions for 64bit-arch."
-"<li>Set the url resolver to escape illegal chars instead of stripping."
-"<li>Big html.cc cleanup. New classes, form API, source split."
-"<li>Added int32_t, EAI_NODATA and iconv tests for FreeBSD."
-"<li>Replaced the findtext dialog with an in-window widget!"
+"<li>Add additional size checks for images.\n"
+"<li>Added support for css colors of the form rgb(255, 255, 255).\n"
+"<li>Added the 'nop' keybinding (nop = NO_OPERATION; cancels a default hook).\n"
+"<li>Added 'stop' key action (not bound by default).\n"
+"<li>Reduced 'warning: ignoring return value of ...'\n"
 "</ul>\n"
-"Dw2:<br>\n"
+"2.1:<br>\n"
 "<ul>\n"
-"<li>Enabled clipped redraws (avoids some flickering)."
-"<li>Added combination of drawing rectangles into a larger one."
-"<li>Made getWidgetAtPoint() a virtual method of widget and implemented a"
-"   custom one for TextBlock, reducing CPU usage on pages full of links."
-"<li>Set FltkViewBase::draw to intersect with view area for expose."
-"<li>Added double buffering for partial redraws!"
-"<li>Reduced memory usage in 30% by reusing styles, reducing the size"
-"   of struct Content, and not preallocating in SimpleVector. !"
-"<li>Moved highlighting information from struct Word into Textblock"
-"   to save memory."
-"<li>Reduced memory usage 10% with a custom memory handler in Textblock."
-"<li>Implemented selection of multibyte glyphs (UTF-8)."
-"<li>Fixed a slithery BUG in lout::misc::Stringbuffer."
-"<li>Added 'enter' and 'leave' signals into class Resource."
-"<li>Enabled mouse wheel scrolling."
-"<li>Added setDeleteCallback(DW_Callback_t func, void *data) to widget."
-"   This allows to hook a callback when the widget is destroyed."
-"<li>Changed the table-apportion algorithms + bug fixes. Big work!"
+"<li>Implemented basic CSS infrastructure!\n"
+"<li>Read user style from ~/.dillo/style.css.\n"
+"<li>Added configurable keybindings! (in ~/.dillo/keysrc)\n"
+"<li>Implemented \"search previous\" in string searches.\n"
+"<li>Ported the command line interface from dillo1\n"
+"<li>Set middle click to submit in a new TAB. (Helps to keep form data!)\n"
+"<li>Implemented Basic authentication!\n"
+"<li>Implemented a close-tab button for the GUI.\n"
+"<li>Implemented a tools menu.\n"
+"<li>Added dillo(1) man page.\n"
+"<li>Added \"font_max_size\", \"font_min_size\" dillorc options.\n"
+"<li>Added instant client-side redirects (aka. zero-delay META refresh).\n"
+"<li>Proxy support for HTTPS.\n"
+"<li>Updated the URL resolver to comply with RFC-3986.\n"
+"<li>Fixed Bookmarks modify's HTML so it wraps nicely on handhelds.\n"
+"<li>Made cookierc parsing more robust.\n"
+"<li>Fix: recover page focus when clicking outside of a widget.\n"
+"<li>Added support for the Q element. BUG#343\n"
+"<li>Added a right-click menu to form controls (show hiddens, submit, reset)\n"
+"<li>Added the \"http_language\" dillorc option for setting HTTP's Accept-Language.\n"
+"<li>Replace image loading button and page menu option with a tools menu option.\n"
+"<li>Implemented the \"overline\" text-decoration.\n"
+"<li>Enhanced and cleaned up text decorations for SUB and SUP.\n"
+"<li>Added \"View Stylesheets\" to the page menu.\n"
+"<li>System config files have moved to sysconfdir/dillo/\n"
+"<li>Allowed compilation with older machines by removing a few C99isms.\n"
+"<li>Switched SSL-enabled to configure.in (./configure --enable-ssl).\n"
+"<li>Removed redundant caller NULL checks already in the API.\n"
+"<li>Added use of inttypes.h when stdint.h isn't found.\n"
+"<li>Made the parser recognize \"[^ ]/>\"-terminated XML elements.\n"
+"<li>Brought in Sebastian's CSS parser from dillo-0.8.0-css-3.\n"
+"<li>Support CSS @import directive.\n"
+"<li>Improved CSS selector matching performance using hash tables.\n"
+"<li>Added support for descendant and child selectors.\n"
+"<li>Support selector specificity.\n"
+"<li>Replace bg_color dillorc option.\n"
+"<li>Remove text_color, link_color, and force_my_colors dillorc options.\n"
+"<li>Replace visited_color dillorc option.\n"
+"<li>Allow negative values for specific CSS properties only.\n"
+"<li>Disable negative margins for now as dw/* does not support them yet.\n"
+"<li>Disable form widgets while stylesheets are loading.\n"
+"<li>Implement --xid command line option (used by claws mail client).\n"
+"<li>Added the \"middle_click_drags_page\" dillorc option.\n"
+"<li>Set the File menu label to hide when the File menu-button is shown.\n"
+"<li>Made a big cleanup of cache.c WRT charset decoding (fixes bugs).\n"
+"<li>Made an extensive cleanup/fixup of the whole image handling process.\n"
+"<li>Fixed handling of META's content-type with no MIME type (e.g. only charset).\n"
+"<li>Added support for a quoted URL in META refresh.\n"
+"<li>Updated the GPL copyright note in the source files.\n"
 "</ul>\n"
 "  </table>\n"
 "</table>\n"
@@ -355,16 +352,16 @@ const char *AboutSplash=
 "<ul>\n"
 " <li> There's a\n"
 "   <a href='http://www.dillo.org/dillorc'>dillorc</a>\n"
-"   (readable  config)  file within the tarball; It is well commented\n"
+"   (readable  config)  file within the tarball; It is well-commented\n"
 "   and  has  plenty  of  options to customize dillo, so <STRONG>copy\n"
 "   it</STRONG>  to  your  <STRONG>~/.dillo/</STRONG>  directory, and\n"
-"   modify to your taste.\n"
-" <li> There's documentation for developers in the <CODE>/doc</CODE>\n"
+"   modify it to your taste.\n"
+" <li> Documentation for developers is in the <CODE>/doc</CODE>\n"
 "   dir  within  the  tarball;  you can find directions on everything\n"
 "   else at the home page.\n"
-" <li> Dillo has context sensitive menus using the\n"
-"   right mouse button (available on pages, links, images,\n"
-"   the Back and Forward buttons, and bug meter).\n"
+" <li> The right mouse button brings up a context-sensitive menu\n"
+"   (available on pages, links, images, forms, the Back and Forward buttons,\n"
+"    and the bug meter).\n"
 " <li> Dillo behaves very nicely when browsing local files, images, and HTML.\n"
 "   It's also very good for Internet searching.\n"
 " <li> This release is mainly intended <strong>for developers</strong>\n"
