@@ -21,13 +21,10 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>           /* for errno */
 
-#include <stdio.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
@@ -353,7 +350,7 @@ static int Dpi_check_uds(char *uds_name)
       strncpy(pun.sun_path, uds_name, sizeof (pun.sun_path));
 
       if ((SockFD = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1 ||
-          connect(SockFD, (void*)&pun, D_SUN_LEN(&pun)) == -1) {
+          connect(SockFD, (struct sockaddr *) &pun, D_SUN_LEN(&pun)) == -1) {
          MSG("Dpi_check_uds: %s %s\n", dStrerror(errno), uds_name);
       } else {
          Dpi_close_fd(SockFD);
@@ -371,7 +368,7 @@ static char *Dpi_get_dpid_uds_dir(void)
 {
    FILE *in;
    char *saved_name_filename;    /*  :)  */
-   char dpid_uds_dir[256], *p = NULL;
+   char buf[256], *dpid_uds_dir, *p = NULL, *nl;
 
    saved_name_filename =
       dStrconcat(dGethomedir(), "/.dillo/dpi_socket_dir", NULL);
@@ -379,14 +376,16 @@ static char *Dpi_get_dpid_uds_dir(void)
    dFree(saved_name_filename);
 
    if (in != NULL) {
-      fgets(dpid_uds_dir, 256, in);
+      dpid_uds_dir = fgets(buf, sizeof(buf), in);
       fclose(in);
-      if ((p = strchr(dpid_uds_dir, '\n'))) {
-         *p = 0;
-      }
-      if (access(dpid_uds_dir, F_OK) == 0) {
-         p = dStrdup(dpid_uds_dir);
-         _MSG("Dpi_get_dpid_uds_dir:: %s\n", p);
+      if (dpid_uds_dir) {
+         if ((nl = strchr(dpid_uds_dir, '\n'))) {
+            *nl = 0;
+         }
+         if (access(dpid_uds_dir, F_OK) == 0) {
+            p = dStrdup(dpid_uds_dir);
+            _MSG("Dpi_get_dpid_uds_dir:: %s\n", p);
+         }
       }
    }
 
@@ -497,7 +496,7 @@ static char *Dpi_get_server_uds_name(const char *server_name)
       _MSG("dpid_uds_name = [%s]\n", dpid_uds_name);
       strncpy(dpid.sun_path, dpid_uds_name, sizeof(dpid.sun_path));
 
-      if (connect(sock, (struct sockaddr *) &dpid, sizeof(dpid)) == -1)
+      if (connect(sock, (struct sockaddr *) &dpid, D_SUN_LEN(&dpid)) == -1)
          perror("connect");
       /* ask dpid to check the server plugin and send its UDS name back */
       request = a_Dpip_build_cmd("cmd=%s msg=%s", "check_server", server_name);
@@ -717,7 +716,6 @@ void a_Dpi_bye_dpid()
 
    srs_name = Dpi_get_dpid_uds_name();
    sun_path_len = sizeof(sa.sun_path);
-   addr_len = sizeof(sa);
 
    sa.sun_family = AF_LOCAL;
 
@@ -725,12 +723,15 @@ void a_Dpi_bye_dpid()
       MSG("a_Dpi_bye_dpid: %s\n", dStrerror(errno));
    }
    strncpy(sa.sun_path, srs_name, sizeof (sa.sun_path));
+   addr_len = D_SUN_LEN(&sa);
    if (connect(new_socket, (struct sockaddr *) &sa, addr_len) == -1) {
       MSG("a_Dpi_bye_dpid: %s\n", dStrerror(errno));
       MSG("%s\n", sa.sun_path);
    }
    DpiBye_cmd = a_Dpip_build_cmd("cmd=%s", "DpiBye");
-   (void) write(new_socket, DpiBye_cmd, strlen(DpiBye_cmd));
+   if (write(new_socket, DpiBye_cmd, strlen(DpiBye_cmd)) !=
+       (ssize_t) strlen(DpiBye_cmd))
+      MSG_WARN("Failed to send DpiBye\n");
    dFree(DpiBye_cmd);
    Dpi_close_fd(new_socket);
 }
