@@ -1,16 +1,15 @@
 /*
  * File: mime.c
  *
- * Copyright (C) 2000 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  */
 
 #include "mime.h"
-#include "../msg.h"
 #include "../list.h"
 
 
@@ -23,10 +22,10 @@ typedef struct {
 /*
  *  Local data
  */
-static gint MimeMinItemsSize = 0, MimeMinItemsMax = 8;
+static int MimeMinItemsSize = 0, MimeMinItemsMax = 8;
 static MimeItem_t *MimeMinItems = NULL;
 
-static gint MimeMajItemsSize = 0, MimeMajItemsMax = 8;
+static int MimeMajItemsSize = 0, MimeMajItemsMax = 8;
 static MimeItem_t *MimeMajItems = NULL;
 
 
@@ -35,7 +34,7 @@ static MimeItem_t *MimeMajItems = NULL;
  * 'Key' is the content-type string that identifies the MIME type
  * 'Method' is the function that handles it
  */
-static gint Mime_add_minor_type(const char *Key, Viewer_t Method)
+static int Mime_add_minor_type(const char *Key, Viewer_t Method)
 {
    a_List_add(MimeMinItems, MimeMinItemsSize, MimeMinItemsMax);
    MimeMinItems[MimeMinItemsSize].Name = Key;
@@ -49,7 +48,7 @@ static gint Mime_add_minor_type(const char *Key, Viewer_t Method)
  * 'Key' is the content-type string that identifies the MIME type
  * 'Method' is the function that handles it
  */
-static gint Mime_add_major_type(const char *Key, Viewer_t Method)
+static int Mime_add_major_type(const char *Key, Viewer_t Method)
 {
    a_List_add(MimeMajItems, MimeMajItemsSize, MimeMajItemsMax);
    MimeMajItems[MimeMajItemsSize].Name = Key;
@@ -62,13 +61,13 @@ static gint Mime_add_major_type(const char *Key, Viewer_t Method)
  * Search the list of specific MIME viewers, for a Method that matches 'Key'
  * 'Key' is the content-type string that identifies the MIME type
  */
-static Viewer_t Mime_minor_type_fetch(const char *Key, guint Size)
+static Viewer_t Mime_minor_type_fetch(const char *Key, uint_t Size)
 {
-   gint i;
+   int i;
 
    if (Size) {
       for ( i = 0; i < MimeMinItemsSize; ++i )
-         if ( g_strncasecmp(Key, MimeMinItems[i].Name, Size) == 0 )
+         if (dStrncasecmp(Key, MimeMinItems[i].Name, Size) == 0)
             return MimeMinItems[i].Data;
    }
    return NULL;
@@ -78,13 +77,13 @@ static Viewer_t Mime_minor_type_fetch(const char *Key, guint Size)
  * Search the list of major MIME viewers, for a Method that matches 'Key'
  * 'Key' is the content-type string that identifies the MIME type
  */
-static Viewer_t Mime_major_type_fetch(const char *Key, guint Size)
+static Viewer_t Mime_major_type_fetch(const char *Key, uint_t Size)
 {
-   gint i;
+   int i;
 
    if (Size) {
       for ( i = 0; i < MimeMajItemsSize; ++i )
-         if ( g_strncasecmp(Key, MimeMajItems[i].Name, Size) == 0 )
+         if (dStrncasecmp(Key, MimeMajItems[i].Name, Size) == 0)
             return MimeMajItems[i].Data;
    }
    return NULL;
@@ -97,16 +96,16 @@ static Viewer_t Mime_major_type_fetch(const char *Key, guint Size)
 void a_Mime_init()
 {
 #ifdef ENABLE_GIF
-   Mime_add_minor_type("image/gif", a_Gif_image);
+   Mime_add_minor_type("image/gif", a_Dicache_gif_image);
 #endif
 #ifdef ENABLE_JPEG
-   Mime_add_minor_type("image/jpeg", a_Jpeg_image);
-   Mime_add_minor_type("image/pjpeg", a_Jpeg_image);
-   Mime_add_minor_type("image/jpg", a_Jpeg_image);
+   Mime_add_minor_type("image/jpeg", a_Dicache_jpeg_image);
+   Mime_add_minor_type("image/pjpeg", a_Dicache_jpeg_image);
+   Mime_add_minor_type("image/jpg", a_Dicache_jpeg_image);
 #endif
 #ifdef ENABLE_PNG
-   Mime_add_minor_type("image/png", a_Png_image);
-   Mime_add_minor_type("image/x-png", a_Png_image);    /* deprecated */
+   Mime_add_minor_type("image/png", a_Dicache_png_image);
+   Mime_add_minor_type("image/x-png", a_Dicache_png_image);    /* deprecated */
 #endif
    Mime_add_minor_type("text/html", a_Html_text);
 
@@ -122,12 +121,12 @@ void a_Mime_init()
  *   On success: a new Dw (and Call and Data properly set).
  *   On failure: NULL (and Call and Data untouched).
  */
-DwWidget *a_Mime_set_viewer(const char *content_type, void *Ptr,
-                            CA_Callback_t *Call, void **Data)
+void *a_Mime_set_viewer(const char *content_type, void *Ptr,
+                        CA_Callback_t *Call, void **Data)
 {
 
    Viewer_t viewer;
-   guint MinSize, MajSize, i;
+   uint_t MinSize, MajSize, i;
    const char *str = content_type;
 
    MajSize = 0;
