@@ -12,13 +12,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 
+#include "utf8.hh"
 #include "msg.h"
 #include "misc.h"
-
+#include "utf8.hh"
 
 /*
  * Escape characters as %XX sequences.
@@ -26,7 +26,7 @@
  */
 char *a_Misc_escape_chars(const char *str, const char *esc_set)
 {
-   static const char *hex = "0123456789ABCDEF";
+   static const char *const hex = "0123456789ABCDEF";
    char *p = NULL;
    Dstr *dstr;
    int i;
@@ -47,33 +47,41 @@ char *a_Misc_escape_chars(const char *str, const char *esc_set)
    return p;
 }
 
-
 #define TAB_SIZE 8
 /*
  * Takes a string and converts any tabs to spaces.
  */
 char *a_Misc_expand_tabs(const char *str, int len)
 {
-   Dstr *New = dStr_new("");
-   int i, j, pos, old_pos;
+   int i = 0, j, pos = 0, old_pos, char_len;
+   uint_t code;
    char *val;
 
-   if (len) {
-      for (pos = 0, i = 0; i < len; i++) {
-         if (str[i] == '\t') {
+   if (memchr(str, '\t', len) == NULL) {
+      val = dStrndup(str, len);
+   } else {
+      Dstr *New = dStr_new("");
+
+      while (i < len) {
+         code = a_Utf8_decode(&str[i], str + len, &char_len);
+
+         if (code == '\t') {
             /* Fill with whitespaces until the next tab. */
             old_pos = pos;
             pos += TAB_SIZE - (pos % TAB_SIZE);
             for (j = old_pos; j < pos; j++)
                dStr_append_c(New, ' ');
          } else {
-            dStr_append_c(New, str[i]);
+            dStr_append_l(New, &str[i], char_len);
             pos++;
          }
+
+         i += char_len;
       }
+
+      val = New->str;
+      dStr_free(New, FALSE);
    }
-   val = New->str;
-   dStr_free(New, FALSE);
    return val;
 }
 
@@ -150,7 +158,7 @@ int a_Misc_get_content_type_from_data(void *Data, size_t Size, const char **PT)
        * All in the above set regard [00-31] as control characters.
        * LATIN1: [7F-9F] unused
        * CP-1251 {7F,98} unused (two characters).
-       * 
+       *
        * We'll use [0-31] as indicators of non-text content.
        * Better heuristics are welcomed! :-) */
 
@@ -165,12 +173,16 @@ int a_Misc_get_content_type_from_data(void *Data, size_t Size, const char **PT)
          if (ch > 190)
             ++non_ascci_text;
       }
-      if (bin_chars == 0) {
+      if (bin_chars == 0 && (non_ascci - non_ascci_text) <= Size/10) {
          /* Let's say text: if "rare" chars are <= 10% */
-         if ((non_ascci - non_ascci_text) <= Size/10)
+         Type = DT_TEXT_PLAIN;
+      } else if (Size > 0) {
+         /* a special check for UTF-8 */
+         Size = a_Utf8_end_of_char(p, Size - 1) + 1;
+         if (a_Utf8_test(p, Size) > 0)
             Type = DT_TEXT_PLAIN;
       }
-      if (Size == 256)
+      if (Size >= 256)
          st = 0;
    }
 
@@ -185,6 +197,7 @@ void a_Misc_parse_content_type(const char *str, char **major, char **minor,
                                char **charset)
 {
    const char *s;
+   bool_t is_text;
 
    if (major)
       *major = NULL;
@@ -198,6 +211,7 @@ void a_Misc_parse_content_type(const char *str, char **major, char **minor,
    for (s = str; isalnum(*s) || (*s == '-'); s++);
    if (major)
       *major = dStrndup(str, s - str);
+   is_text = (s - str == 4) && !dStrncasecmp(str, "text", 4);
 
    if (*s == '/') {
       for (str = ++s; isalnum(*s) || (*s == '-'); s++);
@@ -205,7 +219,8 @@ void a_Misc_parse_content_type(const char *str, char **major, char **minor,
          *minor = dStrndup(str, s - str);
    }
 
-   if (charset && *s) {
+   if (is_text && charset && *s) {
+      /* charset parameter is defined for text media type (RFC 2046) */
       const char terminators[] = " ;\t";
       const char key[] = "charset";
 
@@ -346,9 +361,9 @@ int a_Misc_parse_geometry(char *str, int *x, int *y, int *w, int *h)
  */
 char *a_Misc_encode_base64(const char *in)
 {
-   static const char *base64_hex = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                                   "abcdefghijklmnopqrstuvwxyz"
-                                   "0123456789+/";
+   static const char *const base64_hex = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                                         "abcdefghijklmnopqrstuvwxyz"
+                                         "0123456789+/";
    char *out = NULL;
    int len, i = 0;
 
