@@ -4,6 +4,7 @@
  *
  * Geoff Lane nov 1999 zzassgl@twirl.mcc.ac.uk
  * Luca Rota, Jorge Arellano Cid, Eric Gaudet 2000
+ * Jorge Arellano Cid 2009
  *
  * "PNG: The Definitive Guide" by Greg Roelofs, O'Reilly
  * ISBN 1-56592-542-4
@@ -12,37 +13,29 @@
 #include <config.h>
 #ifdef ENABLE_PNG
 
-#include <stdio.h>
-#include <string.h>
 #include <stdlib.h> /* For abort() */
 
-#include <zlib.h>
-
 #ifdef HAVE_LIBPNG_PNG_H
 #include <libpng/png.h>
 #else
 #include <png.h>
 #endif
 
-#include <gtk/gtk.h>
 #include "msg.h"
-#include "image.h"
-#include "web.h"
+#include "image.hh"
 #include "cache.h"
 #include "dicache.h"
-#include "prefs.h"
-
-#define DEBUG_LEVEL 6
-#include "debug.h"
 
 enum prog_state {
    IS_finished, IS_init, IS_nextdata
 };
 
+#if 0
 static char *prog_state_name[] =
 {
    "IS_finished", "IS_init", "IS_nextdata"
 };
+#endif
 
 /*
  * This holds the data that must be saved between calls to this module.
@@ -69,18 +62,19 @@ typedef
 struct _DilloPng {
    DilloImage *Image;           /* Image meta data */
    DilloUrl *url;               /* Primary Key for the dicache */
-   gint version;                /* Secondary Key for the dicache */
+   int version;                 /* Secondary Key for the dicache */
 
    double display_exponent;     /* gamma correction */
-   gulong width;                /* png image width */
-   gulong height;               /* png image height */
+   ulong_t width;               /* png image width */
+   ulong_t height;              /* png image height */
    png_structp png_ptr;         /* libpng private data */
    png_infop info_ptr;          /* libpng private info */
-   guchar *image_data;          /* decoded image data    */
-   guchar **row_pointers;       /* pntr to row starts    */
+   uchar_t *image_data;         /* decoded image data    */
+   uchar_t **row_pointers;      /* pntr to row starts    */
    jmp_buf jmpbuf;              /* png error processing */
-   gint error;                  /* error flag */
-   gint rowbytes;               /* No. bytes in image row */
+   int error;                   /* error flag */
+   png_uint_32 previous_row;
+   int rowbytes;                /* No. bytes in image row */
    short passes;
    short channels;              /* No. image channels */
 
@@ -93,13 +87,13 @@ struct _DilloPng {
  * ipbuf    ipbufstart                            ipbufsize
  */
 
-   guchar *ipbuf;               /* image data in buffer */
-   gint ipbufstart;             /* first valid image byte */
-   gint ipbufsize;              /* size of valid data in */
+   uchar_t *ipbuf;              /* image data in buffer */
+   int ipbufstart;              /* first valid image byte */
+   int ipbufsize;               /* size of valid data in */
 
    enum prog_state state;       /* FSM current state  */
 
-   guchar *linebuf;             /* o/p raster data */
+   uchar_t *linebuf;            /* o/p raster data */
 
 } DilloPng;
 
@@ -107,12 +101,13 @@ struct _DilloPng {
 #define BLACK     0
 #define WHITE     255
 
+
 static
 void Png_error_handling(png_structp png_ptr, png_const_charp msg)
 {
    DilloPng *png;
 
-   DEBUG_MSG(6, "Png_error_handling: %s\n", msg);
+   MSG("Png_error_handling: %s\n", msg);
    png = png_get_error_ptr(png_ptr);
 
    png->error = 1;
@@ -125,23 +120,32 @@ static void
 Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
 {
    DilloPng *png;
-   gint color_type;
-   gint bit_depth;
-   gint interlace_type;
-   guint i;
+   int color_type;
+   int bit_depth;
+   int interlace_type;
+   uint_t i;
    double gamma;
 
-   DEBUG_MSG(5, "Png_datainfo_callback:\n");
+   _MSG("Png_datainfo_callback:\n");
 
    png = png_get_progressive_ptr(png_ptr);
-   g_return_if_fail (png != NULL);
+   dReturn_if_fail (png != NULL);
 
    png_get_IHDR(png_ptr, info_ptr, &png->width, &png->height,
                 &bit_depth, &color_type, &interlace_type, NULL, NULL);
 
-   DEBUG_MSG(5, "Png_datainfo_callback: png->width  = %ld\n"
-             "Png_datainfo_callback: png->height = %ld\n",
-             png->width, png->height);
+   /* check max image size */
+   if (png->width <= 0 || png->height <= 0 ||
+       png->width > IMAGE_MAX_AREA / png->height) {
+      MSG("Png_datainfo_callback: suspicious image size request %ldx%ld\n",
+          png->width, png->height);
+      Png_error_handling(png_ptr, "Aborting...");
+      return; /* not reached */
+   }
+
+   _MSG("Png_datainfo_callback: png->width  = %ld\n"
+        "Png_datainfo_callback: png->height = %ld\n",
+        png->width, png->height);
 
    /* we need RGB/RGBA in the end */
    if (color_type == PNG_COLOR_TYPE_PALETTE && bit_depth <= 8) {
@@ -162,7 +166,7 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
    }
 
    /* Get and set gamma information. Beware: gamma correction 2.2 will
-      only work on PC's. todo: select screen gamma correction for other
+      only work on PC's. TODO: select screen gamma correction for other
       platforms. */
    if (png_get_gAMA(png_ptr, info_ptr, &gamma))
       png_set_gamma(png_ptr, 2.2, gamma);
@@ -188,54 +192,62 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
    png->channels = png_get_channels(png_ptr, info_ptr);
 
    /* init Dillo specifics */
-   DEBUG_MSG(5, "Png_datainfo_callback: rowbytes = %d\n"
-             "Png_datainfo_callback: width    = %ld\n"
-             "Png_datainfo_callback: height   = %ld\n",
-             png->rowbytes, png->width, png->height);
+   _MSG("Png_datainfo_callback: rowbytes = %d\n"
+        "Png_datainfo_callback: width    = %ld\n"
+        "Png_datainfo_callback: height   = %ld\n",
+        png->rowbytes, png->width, png->height);
 
-   png->image_data = (guchar *) g_malloc(png->rowbytes * png->height);
-   png->row_pointers = (guchar **) g_malloc(png->height * sizeof(guchar *));
+   png->image_data = (uchar_t *) dMalloc(png->rowbytes * png->height);
+   png->row_pointers = (uchar_t **) dMalloc(png->height * sizeof(uchar_t *));
 
    for (i = 0; i < png->height; i++)
       png->row_pointers[i] = png->image_data + (i * png->rowbytes);
 
-   png->linebuf = g_malloc(3 * png->width);
+   png->linebuf = dMalloc(3 * png->width);
 
    /* Initialize the dicache-entry here */
    a_Dicache_set_parms(png->url, png->version, png->Image,
-                       png->width, png->height, DILLO_IMG_TYPE_RGB);
+                       (uint_t)png->width, (uint_t)png->height,
+                       DILLO_IMG_TYPE_RGB);
 }
 
 static void
  Png_datarow_callback(png_structp png_ptr, png_bytep new_row,
-                      png_uint_32 row_num, gint pass)
+                      png_uint_32 row_num, int pass)
 {
    DilloPng *png;
-   guint i;
+   uint_t i;
 
    if (!new_row)                /* work to do? */
       return;
 
-   DEBUG_MSG(5, "Png_datarow_callback: row_num = %ld\n", row_num);
+   _MSG("Png_datarow_callback: row_num = %ld\n", row_num);
 
    png = png_get_progressive_ptr(png_ptr);
 
    png_progressive_combine_row(png_ptr, png->row_pointers[row_num], new_row);
 
+   _MSG("png: row_num=%u previous_row=%u\n", row_num, png->previous_row);
+   if (row_num < png->previous_row) {
+      a_Dicache_new_scan(png->url, png->version);
+   }
+   png->previous_row = row_num;
+
    switch (png->channels) {
    case 3:
-      a_Dicache_write(png->Image, png->url, png->version,
-                      png->image_data + (row_num * png->rowbytes), 0, row_num);
+      a_Dicache_write(png->url, png->version,
+                      png->image_data + (row_num * png->rowbytes),
+                      (uint_t)row_num);
       break;
    case 4:
      {
-        /* todo: get the backgound color from the parent
+        /* TODO: get the backgound color from the parent
          * of the image widget -- Livio.                 */
-        gint a, bg_red, bg_green, bg_blue;
-        guchar *pl = png->linebuf;
-        guchar *data = png->image_data + (row_num * png->rowbytes);
+        int a, bg_red, bg_green, bg_blue;
+        uchar_t *pl = png->linebuf;
+        uchar_t *data = png->image_data + (row_num * png->rowbytes);
 
-        /* todo: maybe change prefs.bg_color to `a_Dw_widget_get_bg_color`,
+        /* TODO: maybe change prefs.bg_color to `a_Dw_widget_get_bg_color`,
          * when background colors are correctly implementated */
         bg_blue  = (png->Image->bg_color) & 0xFF;
         bg_green = (png->Image->bg_color>>8) & 0xFF;
@@ -244,12 +256,12 @@ static void
         for (i = 0; i < png->width; i++) {
            a = *(data+3);
 
-           if ( a == 255 ) {
+           if (a == 255) {
               *(pl++) = *(data++);
               *(pl++) = *(data++);
               *(pl++) = *(data++);
               data++;
-           } else if ( a == 0 ) {
+           } else if (a == 0) {
               *(pl++) = bg_red;
               *(pl++) = bg_green;
               *(pl++) = bg_blue;
@@ -261,85 +273,71 @@ static void
               data++;
            }
         }
-        a_Dicache_write(png->Image, png->url, png->version,
-                        png->linebuf, 0, row_num);
+        a_Dicache_write(png->url, png->version, png->linebuf, (uint_t)row_num);
         break;
      }
    default:
-      MSG("Png_datarow_callback: unexpected number of channels = %d\n",
-          png->channels);
+      MSG("Png_datarow_callback: unexpected number of channels=%d pass=%d\n",
+          png->channels, pass);
       abort();
    }
 }
 
-static void
- Png_dataend_callback(png_structp png_ptr, png_infop info_ptr)
+static void Png_dataend_callback(png_structp png_ptr, png_infop info_ptr)
 {
    DilloPng *png;
 
-   DEBUG_MSG(5, "Png_dataend_callback:\n");
+   _MSG("Png_dataend_callback:\n");
+   if (!info_ptr)
+      MSG("Png_dataend_callback: info_ptr = NULL\n");
 
    png = png_get_progressive_ptr(png_ptr);
    png->state = IS_finished;
 }
 
+/*
+ * Free up the resources for this image.
+ */
+static void Png_free(DilloPng *png)
+{
+   _MSG("Png_free: png=%p\n", png);
+
+   dFree(png->image_data);
+   dFree(png->row_pointers);
+   dFree(png->linebuf);
+   if (setjmp(png->jmpbuf))
+      MSG_WARN("PNG: can't destroy read structure\n");
+   else if (png->png_ptr)
+      png_destroy_read_struct(&png->png_ptr, &png->info_ptr, NULL);
+   dFree(png);
+}
 
 /*
- * Op:  Operation to perform.
- *   If (Op == 0)
- *      start or continue processing an image if image data exists.
- *   else
- *       terminate processing, cleanup any allocated memory,
- *       close down the decoding process.
- *
- * Client->CbData  : pointer to previously allocated DilloPng work area.
- *  This holds the current state of the image processing and is saved
- *  across calls to this routine.
- * Client->Buf     : Pointer to data start.
- * Client->BufSize : the size of the data buffer.
- *
- * You have to keep track of where you are in the image data and
- * how much has been processed.
- *
- * It's entirely possible that you will not see the end of the data.  The
- * user may terminate transfer via a Stop button or there may be a network
- * failure.  This means that you can't just wait for all the data to be
- * presented before starting conversion and display.
+ * Finish the decoding process (and free the memory)
  */
-static void Png_callback(int Op, CacheClient_t *Client)
+static void Png_close(DilloPng *png, CacheClient_t *Client)
 {
-   DilloPng *png = Client->CbData;
+   _MSG("Png_close\n");
+   /* Let dicache know decoding is over */
+   a_Dicache_close(png->url, png->version, Client);
+   Png_free(png);
+}
 
-   if ( Op ) {
-      /* finished - free up the resources for this image */
-      a_Dicache_close(png->url, png->version, Client);
-      if (png->image_data != NULL)
-         g_free(png->image_data);
-      if (png->row_pointers != NULL)
-         g_free(png->row_pointers);
-      if (png->linebuf != NULL)
-         g_free(png->linebuf);
-      if (setjmp(png->jmpbuf))
-         g_warning("PNG: can't destroy read structure\n");
-      else if (png->png_ptr)
-         png_destroy_read_struct(&png->png_ptr, &png->info_ptr, NULL);
-      g_free(png);
-      return;
-   }
-
-   /* Let's make some sound if we have been called with no data */
-   g_return_if_fail ( Client->Buf != NULL && Client->BufSize > 0 );
-
-   DEBUG_MSG(5, "Png_callback BufSize = %d\n", Client->BufSize);
+/*
+ * Receive and process new chunks of PNG image data
+ */
+static void Png_write(DilloPng *png, void *Buf, uint_t BufSize)
+{
+   dReturn_if_fail ( Buf != NULL && BufSize > 0 );
 
    /* Keep local copies so we don't have to pass multiple args to
     * a number of functions. */
-   png->ipbuf = Client->Buf;
-   png->ipbufsize = Client->BufSize;
+   png->ipbuf = Buf;
+   png->ipbufsize = BufSize;
 
    /* start/resume the FSM here */
    while (png->state != IS_finished && DATASIZE) {
-      DEBUG_MSG(5, "State = %s\n", prog_state_name[png->state]);
+      _MSG("State = %s\n", prog_state_name[png->state]);
 
       switch (png->state) {
       case IS_init:
@@ -349,6 +347,7 @@ static void Png_callback(int Op, CacheClient_t *Client)
          /* check the image signature - DON'T update ipbufstart! */
          if (!png_check_sig(png->ipbuf, DATASIZE)) {
             /* you lied to me about it being a PNG image */
+            MSG_WARN("\"%s\" is not a PNG file.\n", URL_STR(png->url));
             png->state = IS_finished;
             break;
          }
@@ -358,9 +357,9 @@ static void Png_callback(int Op, CacheClient_t *Client)
                            png,
                            (png_error_ptr)Png_error_handling,
                            (png_error_ptr)Png_error_handling);
-         g_return_if_fail (png->png_ptr != NULL);
+         dReturn_if_fail (png->png_ptr != NULL);
          png->info_ptr = png_create_info_struct(png->png_ptr);
-         g_return_if_fail (png->info_ptr != NULL);
+         dReturn_if_fail (png->info_ptr != NULL);
 
          setjmp(png->jmpbuf);
          if (!png->error) {
@@ -375,31 +374,67 @@ static void Png_callback(int Op, CacheClient_t *Client)
          break;
 
       case IS_nextdata:
-         if ( setjmp(png->jmpbuf) ) {
+         if (setjmp(png->jmpbuf)) {
             png->state = IS_finished;
          } else if (!png->error) {
             png_process_data( png->png_ptr,
                               png->info_ptr,
                               png->ipbuf + png->ipbufstart,
-                              DATASIZE);
+                              (png_size_t)DATASIZE);
 
             png->ipbufstart += DATASIZE;
          }
          break;
 
       default:
-         g_warning("PNG decoder: bad state = %d\n", png->state);
+         MSG_WARN("PNG decoder: bad state = %d\n", png->state);
          abort();
       }
    }
 }
 
+/*
+ * Op:  Operation to perform.
+ *   If (Op == 0)
+ *      start or continue processing an image if image data exists.
+ *   else
+ *       terminate processing, cleanup any allocated memory,
+ *       close down the decoding process.
+ *
+ * Client->CbData  : pointer to previously allocated DilloPng work area.
+ *  This holds the current state of the image processing and is kept
+ *  across calls to this routine.
+ * Client->Buf     : Pointer to data start.
+ * Client->BufSize : the size of the data buffer.
+ *
+ * You have to keep track of where you are in the image data and
+ * how much has been processed.
+ *
+ * It's entirely possible that you will not see the end of the data.  The
+ * user may terminate transfer via a Stop button or there may be a network
+ * failure.  This means that you can't just wait for all the data to be
+ * presented before starting conversion and display.
+ */
+void a_Png_callback(int Op, void *data)
+{
+   if (Op == CA_Send) {
+      CacheClient_t *Client = data;
+      Png_write(Client->CbData, Client->Buf, Client->BufSize);
+   } else if (Op == CA_Close) {
+      CacheClient_t *Client = data;
+      Png_close(Client->CbData, Client);
+   } else if (Op == CA_Abort) {
+      Png_free(data);
+   }
+}
+
 /*
  * Create the image state data that must be kept between calls
  */
-static DilloPng *Png_new(DilloImage *Image, DilloUrl *url, gint version)
+void *a_Png_new(DilloImage *Image, DilloUrl *url, int version)
 {
-   DilloPng *png = g_new0(DilloPng, 1);
+   DilloPng *png = dNew0(DilloPng, 1);
+   _MSG("a_Png_new: png=%p\n", png);
 
    png->Image = Image;
    png->url = url;
@@ -412,54 +447,14 @@ static DilloPng *Png_new(DilloImage *Image, DilloUrl *url, gint version)
    png->linebuf = NULL;
    png->image_data = NULL;
    png->row_pointers = NULL;
+   png->previous_row = 0;
 
    return png;
 }
 
-/*
- * MIME handler for "image/png" type
- * (Sets Png_callback or a_Dicache_callback as the cache-client)
- */
-DwWidget *a_Png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
-                      void **Data)
-{
-/*
- * Type: MIME type
- * Ptr:  points to a Web structure
- * Call: Dillo calls this with more data/eod
- * Data: raw image data
- */
+#else /* ENABLE_PNG */
 
-   DilloWeb *web = Ptr;
-   DICacheEntry *DicEntry;
-
-   DEBUG_MSG(5, "a_Png_image: Type = %s\n"
-             "a_Png_image: libpng - Compiled %s; using %s.\n"
-             "a_Png_image: zlib   - Compiled %s; using %s.\n",
-             Type, PNG_LIBPNG_VER_STRING, png_libpng_ver,
-             ZLIB_VERSION, zlib_version);
-
-   if ( !web->Image )
-      web->Image = a_Image_new(0, 0, NULL, NULL, prefs.bg_color);
-
-   /* Add an extra reference to the Image (for dicache usage) */
-   a_Image_ref(web->Image);
-
-   DicEntry = a_Dicache_get_entry(web->url);
-   if ( !DicEntry ) {
-      /* Let's create an entry for this image... */
-      DicEntry = a_Dicache_add_entry(web->url);
-
-      /* ... and let the decoder feed it! */
-      *Data = Png_new(web->Image, DicEntry->url, DicEntry->version);
-      *Call = (CA_Callback_t) Png_callback;
-   } else {
-      /* Let's feed our client from the dicache */
-      a_Dicache_ref(DicEntry->url, DicEntry->version);
-      *Data = web->Image;
-      *Call = (CA_Callback_t) a_Dicache_callback;
-   }
-   return DW_WIDGET (web->Image->dw);
-}
+void *a_Png_new() { return 0; }
+void a_Png_callback() { return; }
 
 #endif /* ENABLE_PNG */
