@@ -1,12 +1,12 @@
 /*
  * File: url.c
  *
- * Copyright (C) 2001 Jorge Arellano Cid <jcid@dillo.org>
- *               2001 Livio Baldini Soares <livio@linux.ime.usp.br>
+ * Copyright (C) 2001 Livio Baldini Soares <livio@linux.ime.usp.br>
+ * Copyright (C) 2001-2009 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  */
 
@@ -18,7 +18,7 @@
  */
 
 /*
- * Regular Expression as given in RFC2396 for URL parsing.
+ * Regular Expression as given in RFC3986 for URL parsing.
  *
  *  ^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?
  *   12            3  4          5       6  7        8 9
@@ -28,41 +28,56 @@
  *  path      = $5
  *  query     = $7
  *  fragment  = $9
+ *
+ *
+ *  RFC-2396 BNF:
+ *
+ *  absoluteURI = scheme ":" (hier_part | opaque_part)
+ *  hier_part   = (net_path | abs_path) ["?" query]
+ *  net_path    = "//" authority[abs_path]
+ *  abs_path    = "/" path_segments
+ *
+ *  Notes:
+ *    - "undefined" means "preceeding separator does not appear".
+ *    - path is never "undefined" though it may be "empty".
  */
 
-
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <glib.h>
 
 #include "url.h"
+#include "msg.h"
 
-/*#define DEBUG_LEVEL 2 */
-#include "debug.h"
+static const char *HEX = "0123456789ABCDEF";
 
+/* URL-field compare methods */
+#define URL_STR_FIELD_CMP(s1,s2) \
+   (s1) && (s2) ? strcmp(s1,s2) : !(s1) && !(s2) ? 0 : (s1) ? 1 : -1
+#define URL_STR_FIELD_I_CMP(s1,s2) \
+   (s1) && (s2) ? dStrcasecmp(s1,s2) : !(s1) && !(s2) ? 0 : (s1) ? 1 : -1
 
 /*
  * Return the url as a string.
- * (initializing 'url_string' camp if necessary)
+ * (initializing 'url_string' field if necessary)
  */
-gchar *a_Url_str(const DilloUrl *u)
+char *a_Url_str(const DilloUrl *u)
 {
    /* Internal url handling IS transparent to the caller */
    DilloUrl *url = (DilloUrl *) u;
 
-   g_return_val_if_fail (url != NULL, NULL);
+   dReturn_val_if_fail (url != NULL, NULL);
 
    if (!url->url_string) {
-      url->url_string = g_string_sized_new(60);
-      g_string_sprintf(
+      url->url_string = dStr_sized_new(60);
+      dStr_sprintf(
          url->url_string, "%s%s%s%s%s%s%s%s%s%s",
          url->scheme    ? url->scheme : "",
          url->scheme    ? ":" : "",
          url->authority ? "//" : "",
          url->authority ? url->authority : "",
-         (url->path && url->path[0] != '/' && url->authority) ? "/" : "",
+         // (url->path && url->path[0] != '/' && url->authority) ? "/" : "",
+         (url->authority && (!url->path || *url->path != '/')) ? "/" : "",
          url->path      ? url->path : "",
          url->query     ? "?" : "",
          url->query     ? url->query : "",
@@ -75,21 +90,33 @@ gchar *a_Url_str(const DilloUrl *u)
 
 /*
  * Return the hostname as a string.
- * (initializing 'hostname' and 'port' camps if necessary)
+ * (initializing 'hostname' and 'port' fields if necessary)
  * Note: a similar approach can be taken for user:password auth.
  */
-const gchar *a_Url_hostname(const DilloUrl *u)
+const char *a_Url_hostname(const DilloUrl *u)
 {
-   gchar *p;
+   char *p;
    /* Internal url handling IS transparent to the caller */
    DilloUrl *url = (DilloUrl *) u;
 
    if (!url->hostname && url->authority) {
-      if ((p = strchr(url->authority, ':'))) {
-         url->port = strtol(p + 1, NULL, 10);
-         url->hostname = g_strndup(url->authority,(guint)(p - url->authority));
-      } else
-         url->hostname = url->authority;
+      if (url->authority[0] == '[' && (p = strchr(url->authority, ']'))) {
+         /* numeric ipv6 address, strip the brackets */
+         url->hostname = dStrndup(url->authority + 1,
+                                  (uint_t)(p - url->authority - 1));
+         if ((p = strchr(p, ':'))) {
+            url->port = strtol(p + 1, NULL, 10);
+         }
+      } else {
+         /* numeric ipv4 or hostname */
+         if ((p = strchr(url->authority, ':'))) {
+            url->port = strtol(p + 1, NULL, 10);
+            url->hostname = dStrndup(url->authority,
+                                     (uint_t)(p - url->authority));
+         } else {
+            url->hostname = url->authority;
+         }
+      }
    }
 
    return url->hostname;
@@ -99,20 +126,19 @@ const gchar *a_Url_hostname(const DilloUrl *u)
  *  Create a DilloUrl object and initialize it.
  *  (buffer, scheme, authority, path, query and fragment).
  */
-static DilloUrl *Url_object_new(const gchar *uri_str)
+static DilloUrl *Url_object_new(const char *uri_str)
 {
    DilloUrl *url;
-   gchar *s, *p;
+   char *s, *p;
 
-   g_return_val_if_fail (uri_str != NULL, NULL);
+   dReturn_val_if_fail (uri_str != NULL, NULL);
 
-   url = g_new0(DilloUrl, 1);
+   url = dNew0(DilloUrl, 1);
 
    /* remove leading & trailing space from buffer */
-   for (p = (gchar *)uri_str; isspace(*p); ++p);
-   url->buffer = g_strchomp(g_strdup(p));
+   url->buffer = dStrstrip(dStrdup(uri_str));
 
-   s = (gchar *) url->buffer;
+   s = (char *) url->buffer;
    p = strpbrk(s, ":/?#");
    if (p && p[0] == ':' && p > s) {                /* scheme */
       *p = 0;
@@ -149,6 +175,7 @@ static DilloUrl *Url_object_new(const gchar *uri_str)
       s = p + 1;
       url->query = s;
       p = strpbrk(s, "#");
+      url->flags |= URL_Get;
    }
    if (p && p[0] == '#') {                         /* fragment */
       *p = 0;
@@ -161,31 +188,32 @@ static DilloUrl *Url_object_new(const gchar *uri_str)
 
 /*
  *  Free a DilloUrl
+ *  Do nothing if the argument is NULL
  */
 void a_Url_free(DilloUrl *url)
 {
    if (url) {
       if (url->url_string)
-         g_string_free(url->url_string, TRUE);
+         dStr_free(url->url_string, TRUE);
       if (url->hostname != url->authority)
-         g_free((gchar *)url->hostname);
-      g_free((gchar *)url->buffer);
-      g_free((gchar *)url->data);
-      g_free((gchar *)url->alt);
-      g_free(url);
+         dFree((char *)url->hostname);
+      dFree((char *)url->buffer);
+      dStr_free(url->data, 1);
+      dFree((char *)url->alt);
+      dFree(url);
    }
 }
 
 /*
- * Resolve the URL as RFC2396 suggests.
+ * Resolve the URL as RFC3986 suggests.
  */
-static GString *Url_resolve_relative(const gchar *RelStr,
-                                     DilloUrl *BaseUrlPar,
-                                     const gchar *BaseStr)
+static Dstr *Url_resolve_relative(const char *RelStr,
+                                  DilloUrl *BaseUrlPar,
+                                  const char *BaseStr)
 {
-   gchar *p, *s, *e;
-   gint i;
-   GString *SolvedUrl, *Path;
+   char *p, *s, *e;
+   int i;
+   Dstr *SolvedUrl, *Path;
    DilloUrl *RelUrl, *BaseUrl = NULL;
 
    /* parse relative URL */
@@ -198,65 +226,64 @@ static GString *Url_resolve_relative(const gchar *RelStr,
       BaseUrl = Url_object_new(BaseStr);
    }
 
-   SolvedUrl = g_string_sized_new(64);
-   Path = g_string_sized_new(64);
+   SolvedUrl = dStr_sized_new(64);
+   Path = dStr_sized_new(64);
 
-   /* path empty && scheme, authority and query undefined */
-   if (!RelUrl->path && !RelUrl->scheme &&
-       !RelUrl->authority && !RelUrl->query) {
-      g_string_append(SolvedUrl, BaseStr);
+   /* path empty && scheme and authority undefined */
+   if (!RelUrl->path && !RelUrl->scheme && !RelUrl->authority) {
+      dStr_append(SolvedUrl, BaseStr);
+      if ((p = strchr(SolvedUrl->str, '#')))
+         dStr_truncate(SolvedUrl, p - SolvedUrl->str);
 
+      if (RelUrl->query) {                        /* query */
+         if (BaseUrl->query)
+            dStr_truncate(SolvedUrl, BaseUrl->query - BaseUrl->buffer - 1);
+         dStr_append_c(SolvedUrl, '?');
+         dStr_append(SolvedUrl, RelUrl->query);
+      }
       if (RelUrl->fragment) {                    /* fragment */
-         if (BaseUrl->fragment)
-            g_string_truncate(SolvedUrl, BaseUrl->fragment-BaseUrl->buffer-1);
-         g_string_append_c(SolvedUrl, '#');
-         g_string_append(SolvedUrl, RelUrl->fragment);
+         dStr_append_c(SolvedUrl, '#');
+         dStr_append(SolvedUrl, RelUrl->fragment);
       }
       goto done;
 
    } else if (RelUrl->scheme) {                  /* scheme */
-      g_string_append(SolvedUrl, RelStr);
+      dStr_append(SolvedUrl, RelStr);
       goto done;
 
    } else if (RelUrl->authority) {               /* authority */
-      /* Set the Path buffer and goto "STEP 7"; */
+      // Set the Path buffer and goto "STEP 7";
       if (RelUrl->path)
-         g_string_append(Path, RelUrl->path);
-
-   } else if (RelUrl->path && RelUrl->path[0] == '/') {   /* path */
-      g_string_append(Path, RelUrl->path);
+         dStr_append(Path, RelUrl->path);
 
    } else {
-      /* solve relative path */
-      if (BaseUrl->path) {
-         g_string_append(Path, BaseUrl->path);
-         for (i = Path->len; --i >= 0 && Path->str[i] != '/'; );
+      if (RelUrl->path && RelUrl->path[0] == '/') {   /* absolute path */
+         ; /* Ignore BaseUrl path */
+      } else if (BaseUrl->path) {                     /* relative path */
+         dStr_append(Path, BaseUrl->path);
+         for (i = Path->len; --i >= 0 && Path->str[i] != '/'; ) ;
          if (Path->str[i] == '/')
-            g_string_truncate(Path, ++i);
+            dStr_truncate(Path, ++i);
       }
       if (RelUrl->path)
-         g_string_append(Path, RelUrl->path);
+         dStr_append(Path, RelUrl->path);
 
-      /* erase "./" */
+      // erase "./"
       while ((p=strstr(Path->str, "./")) &&
              (p == Path->str || p[-1] == '/'))
-         g_string_erase(Path, p - Path->str, 2);
-      /* erase last "." */
+         dStr_erase(Path, p - Path->str, 2);
+      // erase last "."
       if (Path->len && Path->str[Path->len - 1] == '.' &&
           (Path->len == 1 || Path->str[Path->len - 2] == '/'))
-         g_string_truncate(Path, Path->len - 1);
+         dStr_truncate(Path, Path->len - 1);
 
-      /* erase "<segment>/../" and "<segment>/.." */
+      // erase "<segment>/../" and "<segment>/.."
       s = p = Path->str;
       while ( (p = strstr(p, "/..")) != NULL ) {
-         if ((p[3] == '/' || !p[3]) && (p - s)) { /*  "/../" | "/.."  */
-
-            for (e = p + 3 ; p[-1] != '/' && p > s; --p);
-            if (p[0] != '.' || p[1] != '.' || p[2] != '/') {
-               g_string_erase(Path, p - Path->str, e - p + (*e != 0));
-               p -= (p > Path->str);
-            } else
-               p = e;
+         if (p[3] == '/' || !p[3]) { //  "/../" | "/.."
+            for (e = p + 3 ; p > s && p[-1] != '/'; --p) ;
+            dStr_erase(Path, p - Path->str, e - p + (p > s && *e != 0));
+            p -= (p > Path->str);
          } else
             p += 3;
       }
@@ -267,40 +294,40 @@ static GString *Url_resolve_relative(const gchar *RelStr,
 
    /* scheme */
    if (BaseUrl->scheme) {
-      g_string_append(SolvedUrl, BaseUrl->scheme);
-      g_string_append_c(SolvedUrl, ':');
+      dStr_append(SolvedUrl, BaseUrl->scheme);
+      dStr_append_c(SolvedUrl, ':');
    }
 
    /* authority */
    if (RelUrl->authority) {
-      g_string_append(SolvedUrl, "//");
-      g_string_append(SolvedUrl, RelUrl->authority);
+      dStr_append(SolvedUrl, "//");
+      dStr_append(SolvedUrl, RelUrl->authority);
    } else if (BaseUrl->authority) {
-      g_string_append(SolvedUrl, "//");
-      g_string_append(SolvedUrl, BaseUrl->authority);
+      dStr_append(SolvedUrl, "//");
+      dStr_append(SolvedUrl, BaseUrl->authority);
    }
 
    /* path */
    if ((RelUrl->authority || BaseUrl->authority) &&
        ((Path->len == 0 && (RelUrl->query || RelUrl->fragment)) ||
         (Path->len && Path->str[0] != '/')))
-      g_string_append_c(SolvedUrl, '/'); /* hack? */
-   g_string_append(SolvedUrl, Path->str);
+      dStr_append_c(SolvedUrl, '/'); /* hack? */
+   dStr_append(SolvedUrl, Path->str);
 
    /* query */
    if (RelUrl->query) {
-      g_string_append_c(SolvedUrl, '?');
-      g_string_append(SolvedUrl, RelUrl->query);
+      dStr_append_c(SolvedUrl, '?');
+      dStr_append(SolvedUrl, RelUrl->query);
    }
 
    /* fragment */
    if (RelUrl->fragment) {
-      g_string_append_c(SolvedUrl, '#');
-      g_string_append(SolvedUrl, RelUrl->fragment);
+      dStr_append_c(SolvedUrl, '#');
+      dStr_append(SolvedUrl, RelUrl->fragment);
    }
 
 done:
-   g_string_free(Path, TRUE);
+   dStr_free(Path, TRUE);
    a_Url_free(RelUrl);
    if (BaseUrl != BaseUrlPar)
       a_Url_free(BaseUrl);
@@ -320,68 +347,70 @@ done:
  *     fragment           = "part2"
  *     hostname           = "dillo.sf.net"
  *     port               = 8080
- *     flags              = 0
- *     data               = NULL
+ *     flags              = URL_Get
+ *     data               = Dstr * ("")
  *     alt                = NULL
  *     ismap_url_len      = 0
- *     scrolling_position = 0
  *  }
  *
  *  Return NULL if URL is badly formed.
  */
-DilloUrl* a_Url_new(const gchar *url_str, const gchar *base_url,
-                    gint flags, gint32 posx, gint32 posy)
+DilloUrl* a_Url_new(const char *url_str, const char *base_url)
 {
    DilloUrl *url;
-   gchar *urlstring, *p, *new_str = NULL;
-   GString *SolvedUrl;
-   gint n_ic, n_ic_spc;
+   char *urlstr = (char *)url_str;  /* auxiliar variable, don't free */
+   char *p, *str1 = NULL, *str2 = NULL;
+   Dstr *SolvedUrl;
+   int i, n_ic, n_ic_spc;
 
-   g_return_val_if_fail (url_str != NULL, NULL);
+   dReturn_val_if_fail (url_str != NULL, NULL);
 
    /* Count illegal characters (0x00-0x1F, 0x7F and space) */
-   urlstring = (gchar *)url_str;
    n_ic = n_ic_spc = 0;
-   for (p = urlstring; *p; p++) {
+   for (p = (char*)url_str; *p; p++) {
       n_ic_spc += (*p == ' ') ? 1 : 0;
       n_ic += (*p != ' ' && *p > 0x1F && *p != 0x7F) ? 0 : 1;
    }
    if (n_ic) {
-      /* Strip illegal characters (they could also be encoded).
-       * There's no standard for illegal chars; we chose to strip. */
-      for (p = new_str = g_strdup(urlstring); *urlstring; urlstring++)
-         if (*urlstring > 0x1F && *urlstring != 0x7F && *urlstring != ' ')
-            *p++ = *urlstring;
+      /* Encode illegal characters (they could also be stripped).
+       * There's no standard for illegal chars; we chose to encode. */
+      p = str1 = dNew(char, strlen(url_str) + 2*n_ic + 1);
+      for (i = 0; url_str[i]; ++i)
+         if (url_str[i] > 0x1F && url_str[i] != 0x7F && url_str[i] != ' ')
+            *p++ = url_str[i];
+         else  {
+           *p++ = '%';
+           *p++ = HEX[(url_str[i] >> 4) & 15];
+           *p++ = HEX[url_str[i] & 15];
+         }
       *p = 0;
-      urlstring = new_str;
+      urlstr = str1;
    }
 
    /* let's use a heuristic to set http: as default */
    if (!base_url) {
       base_url = "http:";
-      if (urlstring[0] != '/') {
-         p = strpbrk(urlstring, "/#?:");
+      if (urlstr[0] != '/') {
+         p = strpbrk(urlstr, "/#?:");
          if (!p || *p != ':')
-            urlstring = g_strconcat("//", urlstring, NULL);
-      } else if (urlstring[1] != '/')
-         urlstring = g_strconcat("/", urlstring, NULL);
+            urlstr = str2 = dStrconcat("//", urlstr, NULL);
+      } else if (urlstr[1] != '/')
+         urlstr = str2 = dStrconcat("/", urlstr, NULL);
    }
 
    /* Resolve the URL */
-   SolvedUrl = Url_resolve_relative(urlstring, NULL, base_url);
-   DEBUG_MSG(2, "SolvedUrl = %s\n", SolvedUrl->str);
-   g_return_val_if_fail (SolvedUrl != NULL, NULL);
+   SolvedUrl = Url_resolve_relative(urlstr, NULL, base_url);
+   _MSG("SolvedUrl = %s\n", SolvedUrl->str);
 
    /* Fill url data */
    url = Url_object_new(SolvedUrl->str);
+   url->data = dStr_new("");
    url->url_string = SolvedUrl;
-   url->flags = flags;
-   url->scrolling_position_x = posx;
-   url->scrolling_position_y = posy;
    url->illegal_chars = n_ic;
    url->illegal_chars_spc = n_ic_spc;
 
-   g_free(new_str);
+   dFree(str1);
+   dFree(str2);
    return url;
 }
 
@@ -394,49 +423,53 @@ DilloUrl* a_Url_dup(const DilloUrl *ori)
    DilloUrl *url;
 
    url = Url_object_new(URL_STR_(ori));
-   g_return_val_if_fail (url != NULL, NULL);
+   dReturn_val_if_fail (url != NULL, NULL);
 
-   url->url_string           = g_string_new(URL_STR(ori));
+   url->url_string           = dStr_new(URL_STR(ori));
    url->port                 = ori->port;
    url->flags                = ori->flags;
-   url->data                 = g_strdup(ori->data);
-   url->alt                  = g_strdup(ori->alt);
+   url->alt                  = dStrdup(ori->alt);
    url->ismap_url_len        = ori->ismap_url_len;
-   url->scrolling_position_x = ori->scrolling_position_x;
-   url->scrolling_position_y = ori->scrolling_position_y;
    url->illegal_chars        = ori->illegal_chars;
    url->illegal_chars_spc    = ori->illegal_chars_spc;
-
+   url->data                 = dStr_sized_new(URL_DATA(ori)->len);
+   dStr_append_l(url->data, URL_DATA(ori)->str, URL_DATA(ori)->len);
    return url;
 }
 
 /*
- *  Compare two Url's to check if they are the same.
+ *  Compare two Url's to check if they're the same, or which one is bigger.
+ *
  *  The fields which are compared here are:
  *  <scheme>, <authority>, <path>, <query> and <data>
  *  Other fields are left for the caller to check
  *
- *  Return value: 0 if equal, 1 otherwise
+ *  Return value: 0 if equal, > 0 if A > B, < 0 if A < B.
+ *
+ *  Note: this function defines a sorting order different from strcmp!
  */
-gint a_Url_cmp(const DilloUrl *A, const DilloUrl *B)
+int a_Url_cmp(const DilloUrl *A, const DilloUrl *B)
 {
-   if (!A || !B)
-      return 1;
+   int st;
+
+   dReturn_val_if_fail(A && B, 1);
 
    if (A == B ||
-       (URL_STRCAMP_I_EQ(A->authority, B->authority) &&
-        URL_STRCAMP_EQ(A->path, B->path) &&
-        URL_STRCAMP_EQ(A->query, B->query) &&
-        URL_STRCAMP_EQ(A->data, B->data) &&
-        URL_STRCAMP_I_EQ(A->scheme, B->scheme)))
+       ((st = URL_STR_FIELD_I_CMP(A->authority, B->authority)) == 0 &&
+        (st = strcmp(A->path ? A->path + (*A->path == '/') : "",
+                     B->path ? B->path + (*B->path == '/') : "")) == 0 &&
+        //(st = URL_STR_FIELD_CMP(A->path, B->path)) == 0 &&
+        (st = URL_STR_FIELD_CMP(A->query, B->query)) == 0 &&
+        (st = dStr_cmp(A->data, B->data)) == 0 &&
+        (st = URL_STR_FIELD_I_CMP(A->scheme, B->scheme)) == 0))
       return 0;
-   return 1;
+   return st;
 }
 
 /*
  * Set DilloUrl flags
  */
-void a_Url_set_flags(DilloUrl *u, gint flags)
+void a_Url_set_flags(DilloUrl *u, int flags)
 {
    if (u)
       u->flags = flags;
@@ -445,33 +478,23 @@ void a_Url_set_flags(DilloUrl *u, gint flags)
 /*
  * Set DilloUrl data (like POST info, etc.)
  */
-void a_Url_set_data(DilloUrl *u, gchar *data)
+void a_Url_set_data(DilloUrl *u, Dstr **data)
 {
    if (u) {
-      g_free((gchar *)u->data);
-      u->data = g_strdup(data);
+      dStr_free(u->data, 1);
+      u->data = *data;
+      *data = NULL;
    }
 }
 
 /*
  * Set DilloUrl alt (alternate text to the URL. Used by image maps)
  */
-void a_Url_set_alt(DilloUrl *u, const gchar *alt)
+void a_Url_set_alt(DilloUrl *u, const char *alt)
 {
    if (u) {
-      g_free((gchar *)u->alt);
-      u->alt = g_strdup(alt);
-   }
-}
-
-/*
- * Set DilloUrl scrolling position
- */
-void a_Url_set_pos(DilloUrl *u, gint32 posx, gint32 posy)
-{
-   if (u) {
-      u->scrolling_position_x = posx;
-      u->scrolling_position_y = posy;
+      dFree((char *)u->alt);
+      u->alt = dStrdup(alt);
    }
 }
 
@@ -479,18 +502,18 @@ void a_Url_set_pos(DilloUrl *u, gint32 posx, gint32 posy)
  * Set DilloUrl ismap coordinates
  * (this is optimized for not hogging the CPU)
  */
-void a_Url_set_ismap_coords(DilloUrl *u, gchar *coord_str)
+void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str)
 {
-   g_return_if_fail(u && coord_str);
+   dReturn_if_fail (u && coord_str);
 
-   if ( !u->ismap_url_len ) {
+   if (!u->ismap_url_len) {
       /* Save base-url length (without coords) */
       u->ismap_url_len  = URL_STR_(u) ? u->url_string->len : 0;
       a_Url_set_flags(u, URL_FLAGS(u) | URL_Ismap);
    }
    if (u->url_string) {
-      g_string_truncate(u->url_string, u->ismap_url_len);
-      g_string_append(u->url_string, coord_str);
+      dStr_truncate(u->url_string, u->ismap_url_len);
+      dStr_append(u->url_string, coord_str);
       u->query = u->url_string->str + u->ismap_url_len + 1;
    }
 }
@@ -499,10 +522,10 @@ void a_Url_set_ismap_coords(DilloUrl *u, gchar *coord_str)
  * Given an hex octet (e.g., e3, 2F, 20), return the corresponding
  * character if the octet is valid, and -1 otherwise
  */
-static int Url_decode_hex_octet(const gchar *s)
+static int Url_decode_hex_octet(const char *s)
 {
-   gint hex_value;
-   gchar *tail, hex[3];
+   int hex_value;
+   char *tail, hex[3];
 
    if (s && (hex[0] = s[0]) && (hex[1] = s[1])) {
       hex[2] = 0;
@@ -517,9 +540,9 @@ static int Url_decode_hex_octet(const gchar *s)
  * Parse possible hexadecimal octets in the URI path.
  * Returns a new allocated string.
  */
-gchar *a_Url_decode_hex_str(const gchar *str)
+char *a_Url_decode_hex_str(const char *str)
 {
-   gchar *new_str, *dest;
+   char *new_str, *dest;
    int i, val;
 
    if (!str)
@@ -527,9 +550,9 @@ gchar *a_Url_decode_hex_str(const gchar *str)
 
    /* most cases won't have hex octets */
    if (!strchr(str, '%'))
-      return g_strdup(str);
+      return dStrdup(str);
 
-   dest = new_str = g_new(gchar, strlen(str) + 1);
+   dest = new_str = dNew(char, strlen(str) + 1);
 
    for (i = 0; str[i]; i++) {
       *dest++ = (str[i] == '%' && (val = Url_decode_hex_octet(str+i+1)) >= 0) ?
@@ -537,7 +560,7 @@ gchar *a_Url_decode_hex_str(const gchar *str)
    }
    *dest++ = 0;
 
-   new_str = g_realloc(new_str, sizeof(gchar) * (dest - new_str));
+   new_str = dRealloc(new_str, sizeof(char) * (dest - new_str));
    return new_str;
 }
 
@@ -551,16 +574,15 @@ gchar *a_Url_decode_hex_str(const gchar *str)
  * Note: the content type "application/x-www-form-urlencoded" is used:
  *       i.e., ' ' -> '+' and '\n' -> CR LF (see HTML 4.01, Sec. 17.13.4)
  */
-gchar *a_Url_encode_hex_str(const gchar *str)
+char *a_Url_encode_hex_str(const char *str)
 {
-   static const char *verbatim = "-_.*";
-   static const char *hex = "0123456789ABCDEF";
+   static const char *const verbatim = "-_.*";
    char *newstr, *c;
 
    if (!str)
       return NULL;
 
-   newstr = g_new(char, 6*strlen(str)+1);
+   newstr = dNew(char, 6*strlen(str)+1);
 
    for (c = newstr; *str; str++)
       if ((isalnum(*str) && !(*str & 0x80)) || strchr(verbatim, *str))
@@ -577,8 +599,8 @@ gchar *a_Url_encode_hex_str(const gchar *str)
          *c++ = 'A';
       } else {
          *c++ = '%';
-         *c++ = hex[(*str >> 4) & 15];
-         *c++ = hex[*str & 15];
+         *c++ = HEX[(*str >> 4) & 15];
+         *c++ = HEX[*str & 15];
       }
    *c = 0;
 
@@ -587,15 +609,15 @@ gchar *a_Url_encode_hex_str(const gchar *str)
 
 
 /*
- * RFC-2396 suggests this stripping when "importing" URLs from other media.
+ * RFC-3986 suggests this stripping when "importing" URLs from other media.
  * Strip: "URL:", enclosing < >, and embedded whitespace.
  * (We also strip illegal chars: 00-1F and 7F)
  */
-gchar *a_Url_string_strip_delimiters(const gchar *str)
+char *a_Url_string_strip_delimiters(const char *str)
 {
-   gchar *p, *new_str, *text;
+   char *p, *new_str, *text;
 
-   new_str = text = g_strdup(str);
+   new_str = text = dStrdup(str);
 
    if (new_str) {
       if (strncmp(new_str, "URL:", 4) == 0)
